# -*- coding: UTF-8 -*-

__revision__ = '$Id: __init__.py 1533 2011-02-08 21:04:38Z iznogoud $'

# Copyright (c) 2008-2009 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published byp
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import glob
import os.path
import gutils
import gtk
import xml.dom.minidom
import os

import logging
log = logging.getLogger("Griffith.%s" % __name__)

from sqlalchemy import select, join, outerjoin, and_, or_
import db
from sql import update_whereclause

# detect all plugins:
__all__ = [os.path.basename(x)[:-3] for x in glob.glob("%s/PluginExport*.py" % os.path.dirname(__file__))]

class Base(object):
    description = None
    author = None
    email = None
    version = None
    fields_to_export = ('number', 'o_title', 'title', 'director', 'year')

    def __init__(self, database, locations, parent_window, search_conditions, config):
        self.db = database
        self.config = config
        self.locations = locations
        self.parent_window = parent_window
        self.search_conditions = search_conditions
    
    def initialize(self):
        """Initializes plugin (get all parameters from user, etc.)"""
        return True

    def cleanup(self):
        pass

    def run(self):
        raise NotImplemented
    
    def get_query(self):
        t = db.metadata.tables
        tables = set()
        columns = []

        for i in self.fields_to_export:
            table = 'movies'
            column = i.split('.')
            if len(column) > 1:
                table = column[0]
                column = column[1]
                if table not in t:
                    log.warning("Wrong table name: %s", table)
                    continue
                tables.add(table) # will be used to generate JOIN
            else:
                column = column[0]

            if column in t[table].columns:
                columns.append(t[table].columns[column])
            else:
                log.warning("Wrong field name: %s", i)

        joins = []
        if 'media' in tables:
            joins.append((t['media'], t['movies'].c.medium_id==t['media'].c.medium_id))
        if 'collections' in tables:
            joins.append((t['collections'], t['movies'].c.collection_id==t['collections'].c.collection_id))
        if 'volumes' in tables:
            joins.append((t['volumes'], t['movies'].c.volume_id==t['volumes'].c.volume_id))
        if 'vcodecs' in tables:
            joins.append((t['vcodecs'], t['movies'].c.vcodec_id==t['vcodecs'].c.vcodec_id))

        if joins:
            from_obj = [ outerjoin(t['movies'], *(joins[0])) ]
            for j in joins[1:]:
                from_obj.append(outerjoin(from_obj[-1], *j))
            query = select(columns=columns, bind=self.db.session.bind, from_obj=from_obj, use_labels=True)
        else:
            query = select(columns=columns, bind=self.db.session.bind)

        query = update_whereclause(query, self.search_conditions)

        # save column names (will contain 'movies_title' or 'title' depending on how many tables were requested)
        self.exported_columns = query.columns

        return query

class XmlExportBase(Base):

    def __init__(self, database, locations, parent_window, search_conditions, config):
        Base.__init__(self, database, locations, parent_window, search_conditions, config)
        self.db              = database
        self.config_section  = None
        self.export_name     = ''
        self.filepath        = ''
        self.dirpath         = ''
        self.filename        = 'export.xml'
        self.encoding        = 'windows-1252'
        self.exported_movies = 0
        self.true_value      = 'True'
        self.false_value     = 'False'

    def run(self):
        if self.show_dialog() == True:
            # create xml document
            impl = xml.dom.minidom.getDOMImplementation()
            doc  = impl.createDocument(None, None, None)
            self.export_to_document(doc, doc.documentElement)
            # write xml document to file
            self.export_document_to_file(doc, self.filepath)
            gutils.info(_('%s file has been created.') % self.export_name, self.parent_window)

    def show_dialog(self):
        # shows a file dialog and sets self.filepath
        # derived classes which overwrite this method have also to set self.filepath
        basedir = None
        if not self.config is None and not self.config_section is None:
            basedir = self.config.get('export_dir', None, section=self.config_section)
        if basedir is None:
            filenames = gutils.file_chooser(_('Export a %s document') % self.export_name, action=gtk.FILE_CHOOSER_ACTION_SAVE, \
                buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK), name=self.filename)
        else:
            filenames = gutils.file_chooser(_('Export a %s document') % self.export_name, action=gtk.FILE_CHOOSER_ACTION_SAVE, \
                buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK), name=self.filename, folder=basedir)
        if filenames[0]:
            self.filepath = filenames[0]
            if len(filenames) > 1:
                self.dirpath = filenames[1]
            else:
                self.dirpath = os.path.dirname(self.filepath)
            self.filename = os.path.basename(self.filepath)
            if not self.config is None and self.dirpath and not self.config_section is None:
                self.config.set('export_dir', self.dirpath, section=self.config_section)
                self.config.save()
            overwrite = None
            if os.path.isfile(self.filepath):
                if gutils.question(_('File exists. Do you want to overwrite it?'), self.parent_window):
                    overwrite = True
                else:
                    overwrite = False
                    
            if overwrite == True or overwrite is None:
                return True
        return False

    def export_document_to_file(self, document, filename):
        # write XML to file
        xml = document.toprettyxml(encoding=self.encoding)
        fp = open(filename, 'wb')
        try:
            fp.write(xml)
        finally:
            fp.close()

    def export_to_document(self, document, mainelement):
        table_movies = db.metadata.tables['movies']
        # create object
        columns = [table_movies.c.movie_id, table_movies.c.number,
            table_movies.c.title, table_movies.c.o_title,
            table_movies.c.country, table_movies.c.year,
            table_movies.c.runtime, table_movies.c.classification,
            table_movies.c.genre, table_movies.c.region, table_movies.c.studio,
            table_movies.c.cast, table_movies.c.director,
            table_movies.c.plot, table_movies.c.notes,
            table_movies.c.loaned, table_movies.c.rating,
            table_movies.c.trailer, table_movies.c.image,
            table_movies.c.seen, table_movies.c.media_num,
            table_movies.c.poster_md5, table_movies.c.screenplay,
            table_movies.c.cameraman, table_movies.c.barcode]
        # use outer join to media table to get the name of the media
        columns.append(db.metadata.tables['media'].c.name)
        media_join = outerjoin(db.metadata.tables['movies'], \
            db.metadata.tables['media'], \
            db.metadata.tables['movies'].c.medium_id==db.metadata.tables['media'].c.medium_id)
        # use outer join to collections table to get the name of the collection
        columns.append(db.metadata.tables['collections'].c.name)
        collection_join = media_join.outerjoin( \
            db.metadata.tables['collections'], \
            db.metadata.tables['movies'].c.collection_id==db.metadata.tables['collections'].c.collection_id)
        # use outer join to volumes table to get the name of the volume
        columns.append(db.metadata.tables['volumes'].c.name)
        volume_join = collection_join.outerjoin( \
            db.metadata.tables['volumes'], \
            db.metadata.tables['movies'].c.volume_id==db.metadata.tables['volumes'].c.volume_id)
        # use outer join to volumes table to get the name of the video codec
        columns.append(db.metadata.tables['vcodecs'].c.name)
        vcodec_join = volume_join.outerjoin( \
            db.metadata.tables['vcodecs'], \
            db.metadata.tables['movies'].c.vcodec_id==db.metadata.tables['vcodecs'].c.vcodec_id)
        # use outer join to posters table to get the poster image
        columns.append(db.metadata.tables['posters'].c.data)
        posters_join = vcodec_join.outerjoin( \
            db.metadata.tables['posters'], \
            db.metadata.tables['movies'].c.poster_md5==db.metadata.tables['posters'].c.md5sum)
        # fetch movie data
        moviesquery = select(columns=columns, from_obj=[media_join, collection_join, volume_join, vcodec_join, posters_join], bind=self.db.session.bind, use_labels = True)
        moviesquery = update_whereclause(moviesquery, self.search_conditions)
        self.process_movies(document, mainelement, moviesquery)

    def process_movies(self, document, mainelement, moviesquery):
        # process all movies by query
        self.exported_movies = 0
        keys = moviesquery.c.keys
        for movie in moviesquery.execute().fetchall():
            self.process_movie(document, mainelement, movie, keys)
            self.exported_movies += 1

    def process_movie(self, document, movieelement, movie, keys):
        # process the movie itself
        self.process_movie_values(document, movieelement, movie, keys)
        # process tags
        self.process_movie_tags(document, movieelement, movie)
        # process audio codecs
        self.process_movie_acodecs(document, movieelement, movie)
        # process languages
        self.process_movie_languages(document, movieelement, movie)
        # process subtitles
        self.process_movie_subtitles(document, movieelement, movie)

    def process_movie_values(self, document, movieelement, movie, keys):
        # process movie properties by key
        for key in keys():
            if not key == 'posters_data':
                value = self.convert_value(movie[key])
                self.process_movie_value(document, movieelement, key, value)
        self.process_movie_image(document, movieelement, movie['posters_data'], movie['movies_poster_md5'])

    def process_movie_value(self, document, movieelement, key, value):
        pass

    def process_movie_image(self, document, movieelement, imagedata, imagemd5sum):
        pass

    #
    # export movie tags
    #
    def process_movie_tags(self, document, movieelement, movie):
        # build query for movie tags processing
        tagsquerycolumns = [db.metadata.tables['movie_tag'].c.movie_id,
            db.metadata.tables['movie_tag'].c.tag_id,
            db.metadata.tables['tags'].c.name]
        tag_join = join(db.metadata.tables['movie_tag'], \
            db.metadata.tables['tags'], \
            db.metadata.tables['movie_tag'].c.tag_id==db.metadata.tables['tags'].c.tag_id)
        tagsquery = select(\
            bind=self.db.session.bind, \
            columns = tagsquerycolumns, \
            from_obj = [tag_join], \
            whereclause = db.metadata.tables['movie_tag'].c.movie_id==movie['movies_movie_id'])
        self.process_tags(document, movieelement, tagsquery)
        
    def process_tags(self, document, movieelement, tagsquery):
        # process movie tags by query
        pass

    #
    # export movie audio codecs
    #
    def process_movie_acodecs(self, document, movieelement, movie):
        # build query for audio codec processing
        acodecscolumns = [db.metadata.tables['movie_lang'].c.movie_id,
            db.metadata.tables['movie_lang'].c.type,
            db.metadata.tables['movie_lang'].c.acodec_id,
            db.metadata.tables['acodecs'].c.name]
        acodec_join = join(db.metadata.tables['movie_lang'], \
            db.metadata.tables['acodecs'], \
            db.metadata.tables['movie_lang'].c.acodec_id==db.metadata.tables['acodecs'].c.acodec_id)
        acodecsquery = select(\
            bind=self.db.session.bind, \
            columns = acodecscolumns, \
            from_obj = [acodec_join], \
            whereclause = db.metadata.tables['movie_lang'].c.movie_id==movie['movies_movie_id'])
        self.process_acodecs(document, movieelement, acodecsquery)
        
    def process_acodecs(self, document, movieelement, acodecs):
        # process audio codecs by query
        pass

    #
    # export movie languages
    #
    def process_movie_languages(self, document, movieelement, movie):
        # build query for movie languages processing
        languagecolumns = [db.metadata.tables['movie_lang'].c.movie_id,
            db.metadata.tables['movie_lang'].c.type,
            db.metadata.tables['movie_lang'].c.lang_id,
            db.metadata.tables['movie_lang'].c.acodec_id,
            db.metadata.tables['languages'].c.name,
            db.metadata.tables['acodecs'].c.name]
        language_join = join(db.metadata.tables['movie_lang'], \
            db.metadata.tables['languages'], \
            db.metadata.tables['movie_lang'].c.lang_id==db.metadata.tables['languages'].c.lang_id)
        acodec_join = language_join.join( \
            db.metadata.tables['acodecs'], \
            db.metadata.tables['movie_lang'].c.acodec_id==db.metadata.tables['acodecs'].c.acodec_id)
        languagesquery = select(\
            bind=self.db.session.bind, \
            columns = languagecolumns, \
            from_obj = [language_join, acodec_join], \
            whereclause = and_(db.metadata.tables['movie_lang'].c.movie_id==movie['movies_movie_id'], \
                or_(db.metadata.tables['movie_lang'].c.type==0, or_(db.metadata.tables['movie_lang'].c.type==2, db.metadata.tables['movie_lang'].c.type==None))), \
            use_labels = True)
        self.process_languages(document, movieelement, languagesquery)
        
    def process_languages(self, document, movieelement, languages):
        # process movie languages by query
        pass

    #
    # export movie subtitles
    #
    def process_movie_subtitles(self, document, movieelement, movie):
        # build query for movie subtitles processing
        languagescolumns = [db.metadata.tables['movie_lang'].c.movie_id,
            db.metadata.tables['movie_lang'].c.type,
            db.metadata.tables['movie_lang'].c.lang_id,
            db.metadata.tables['languages'].c.name]
        language_join = join(db.metadata.tables['movie_lang'], \
            db.metadata.tables['languages'], \
            db.metadata.tables['movie_lang'].c.lang_id==db.metadata.tables['languages'].c.lang_id)
        languagesquery = select(\
            bind=self.db.session.bind, \
            columns = languagescolumns, \
            from_obj = [language_join], \
            whereclause = and_(db.metadata.tables['movie_lang'].c.movie_id==movie['movies_movie_id'], db.metadata.tables['movie_lang'].c.type==3))
        self.process_subtitles(document, movieelement, languagesquery)
        
    def process_subtitles(self, document, movieelement, subtitlesquery):
        # process movie subtitles by query
        pass

    #
    # helper methods
    #

    def save_image_to_file(self, imagedata, imagebasename, destinationdir):
        # save image image to destination directory
        # build absolute path from current export directory if necessary
        # create picture directory if necessary
        if not os.path.isabs(destinationdir):
            dst_directory = os.path.abspath(os.path.join(self.dirpath, destinationdir))
        else:
            dst_directory = os.path.abspath(destinationdir)
        if not os.path.exists(dst_directory):
            try:
                os.mkdir(dst_directory)
            except:
                log.error("Can't create %s" % dst_directory)
        dst_filename = os.path.join(dst_directory, imagebasename + '.jpg')
        try:
            f = file(dst_filename, 'wb')
            try:
                f.write(imagedata)
            finally:
                f.close()
        except:
            log.error("Can't save image data to %s" % dst_filename)

    def convert_value(self, value):
        if value is None:
            result = ''
        elif value is True:
            result = self.true_value
        elif value is False:
            result = self.false_value
        else:
            try:
                result = str(value).encode(self.encoding).decode(self.encoding)
            except:
                log.error('Can''t convert value ' + value + ' to codepage ' + self.encoding + '.')
                result = ''
        return result



# -*- coding: UTF-8 -*-

__revision__ = '$Id: main_treeview.py 1572 2011-08-23 18:47:22Z mikej06 $'

# Copyright (c) 2005-2011 Vasco Nunes, Piotr Ożarowski

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os
import gtk
from sqlalchemy import select, desc
from sqlalchemy.sql.expression import Select
import db
import gutils
import sql

log = logging.getLogger("Griffith")

def on_tree_selection(tree_selection, self):
    counts = tree_selection.count_selected_rows()
    treeview_selection_on_event(self, self.widgets['treeview'])
    treemodel, selected = tree_selection.get_selected_rows()
    self.selected_iter = [treemodel.get_iter(path) for path in selected]
    
    if counts == 1:
        # TODO: rewrite me
        self.widgets['menu']['clone'].set_sensitive(True)
        self.widgets['menu']['make_cover'].set_sensitive(True)
        self.widgets['menu']['loan'].set_sensitive(True)
        self.widgets['menu']['edit'].set_sensitive(True)
        self.widgets['popups']['edit'].set_sensitive(True)
        self.widgets['popups']['loan'].set_sensitive(True)
        self.widgets['popups']['clone'].set_sensitive(True)
        self.widgets['popups']['print_cover'].set_sensitive(True)
    else:
        set_details(self)
        # lets make some options unavailable when multiselction is active. for now.
        # TODO: rewrite me
        self.widgets['menu']['clone'].set_sensitive(False)
        self.widgets['menu']['make_cover'].set_sensitive(False)
        self.widgets['menu']['loan'].set_sensitive(False)
        self.widgets['menu']['edit'].set_sensitive(False)
        self.widgets['popups']['edit'].set_sensitive(False)
        self.widgets['popups']['loan'].set_sensitive(False)
        self.widgets['popups']['clone'].set_sensitive(False)
        self.widgets['popups']['print_cover'].set_sensitive(False)
    treeview_clicked(self)

def foreach_treeview_selected(model, path, iter, selected):
    selected.append(model.get_value(iter, 0))

def treeview_selection_on_event(self, treeview, iter = False):
    self.selected = []
    treeview.get_selection().selected_foreach(foreach_treeview_selected, self.selected)
    #return selected

def treeview_clicked(self):
    if self.initialized is False:
        return False
    if len(self.selected) == 1:
        movie = self.db.session.query(db.Movie).filter_by(number=int(self.selected[0])).first()
        if self.widgets['poster_window'].flags() & gtk.VISIBLE == gtk.VISIBLE:
            # poster window is visible
            filename = None
            if movie.poster_md5:
                filename = gutils.get_image_fname(movie.poster_md5, self.db)
            if not filename:
                filename = gutils.get_defaultimage_fname(self)
            self.widgets['big_poster'].set_from_file(filename)
        #for ext in self.extensions:
            #ext.maintree_clicked(treeselection, movie)
        set_details(self, movie)
    else:
        return False

def set_details(self, item=None):#{{{
    if item is None:
        item = {}
    if 'movie_id' in item and item['movie_id']:
        self._movie_id = item['movie_id']
    else:
        self._movie_id = None
    w = self.widgets['movie']

    if 'number' in item and item['number']:
        w['number'].set_text(str(int(item['number'])))
    else:
        w['number'].set_text('')
    if 'title' in item and item['title']:
        w['title'].set_markup("<b><span size='large'>%s</span></b>" % gutils.html_encode(item['title']))
    else:
        w['title'].set_text('')
    if 'o_title' in item and item['o_title']:
        w['o_title'].set_markup("<span size='medium'><i>%s</i></span>" % gutils.html_encode(item['o_title']))
    else:
        w['o_title'].set_text('')
    if 'director' in item and item['director']:
        w['director'].set_markup("<i>%s</i>" % gutils.html_encode(item['director']))
    else:
        w['director'].set_text('')
    if 'plot' in item and item['plot']:
        w['plot'].set_text(str(item['plot']))
    else:
        w['plot'].set_text('')
    if 'year' in item and item['year']:
        w['year'].set_text(str(item['year']))
    else:
        w['year'].set_text('')
    if 'resolution' in item and item['resolution']:
        if self.config.get('use_resolution_alias', True):
            w['resolution'].set_text(item['resolution'])
        elif 'height' in item and item['height'] and 'width' in item and item['width']:
            w['resolution'].set_text("%dx%d" % (item['width'], item['height']))
        else: # fallback to 'resolution'
            w['resolution'].set_text(item['resolution'])
    else:
        w['resolution'].set_text('')
        pass
    if 'runtime' in item and item['runtime']:
        w['runtime'].set_text(str(int(item['runtime'])))
    else:
        w['runtime'].set_text('x')
    if 'cameraman' in item and item['cameraman']:
        w['cameraman'].set_markup("<i>%s</i>" % gutils.html_encode(item['cameraman']))
    else:
        w['cameraman'].set_text('')
    if 'screenplay' in item and item['screenplay']:
        w['screenplay'].set_markup("<i>%s</i>" % gutils.html_encode(item['screenplay']))
    else:
        w['screenplay'].set_text('')
    if 'cast' in item and item['cast']:
        w['cast'].set_text(str(item['cast']))
    else:
        w['cast'].set_text('')
    if 'country' in item and item['country']:
        w['country'].set_markup("<i>%s</i>" % gutils.html_encode(item['country']))
    else:
        w['country'].set_text('')
    if 'genre' in item and item['genre']:
        w['genre'].set_markup("<i>%s</i>" % gutils.html_encode(item['genre']))
    else:
        w['genre'].set_text('')
    if 'cond' in item and item['cond']:
        if str(item['cond']) in [str(i) for i in range(len(self._conditions))]:
            w['condition'].set_markup("<i>%s</i>" % self._conditions[item['cond']])
        else:
            w['condition'].set_text('')
            log.info("Wrong value in 'condition' field (movie_id=%s, cond=%s)" % (item['movie_id'], item['cond']))
    else:
        w['condition'].set_text('')
    if 'region' in item and item['region']:
        if item['region'] < len(self._regions):
            w['region'].set_markup("<i>%s</i>" % gutils.html_encode(self._regions[item['region']]))
            if int(item['region']) < 12:
                self.widgets['tooltips'].set_tip(w['region'], self._regions[int(item['region'])])
        else:
            log.info("Wrong value in 'region' field (movie_id=%s, region=%s)" % (item['movie_id'], item['region']))
            w['region'].set_text('')
            self.widgets['tooltips'].set_tip(w['region'], self._regions[0]) # N/A
    else:
        w['region'].set_text('')
        self.widgets['tooltips'].set_tip(w['region'], self._regions[0]) # N/A
    if 'layers' in item and item['layers']:
        if str(item['layers']) in [str(i) for i in range(len(self._layers))]:
            w['layers'].set_markup("<i>%s</i>" % self._layers[item['layers']])
        else:
            log.info("Wrong value in 'layers' field (movie_id=%s, layers=%s)" % (item['movie_id'], item['layers']))
            w['layers'].set_text('')
    else:
        w['layers'].set_text('')
    if 'color' in item and item['color']:
        if str(item['color']) in [str(i) for i in range(len(self._colors))]:
            w['color'].set_markup("<i>%s</i>" % self._colors[item['color']])
        else:
            log.info("Wrong value in 'color' field (movie_id=%s, color=%s)" % (item['movie_id'], item['color']))
            w['color'].set_markup('')
    else:
        w['color'].set_markup('')
    if 'classification' in item and item['classification']:
        w['classification'].set_markup("<i>%s</i>" % gutils.html_encode(item['classification']))
    else:
        w['classification'].set_text('')
    if 'studio' in item and item['studio']:
        w['studio'].set_markup("<i>%s</i>" % gutils.html_encode(item['studio']))
    else:
        w['studio'].set_text('')
    if 'o_site' in item and item['o_site']:
        self._o_site_url = str(item['o_site'])
        w['go_o_site_button'].set_sensitive(True)
    else:
        self._o_site_url = None
        w['go_o_site_button'].set_sensitive(False)
    if 'site' in item and item['site']:
        self._site_url = str(item['site'])
        w['go_site_button'].set_sensitive(True)
    else:
        self._site_url = None
        w['go_site_button'].set_sensitive(False)
    if 'trailer' in item and item['trailer']:
        self._trailer_url = str(item.trailer)
        w['go_trailer_button'].set_sensitive(True)
    else:
        self._trailer_url = None
        w['go_trailer_button'].set_sensitive(False)
    if 'seen' in item and item['seen'] == True:
        w['seen_icon'].set_from_file(os.path.join(self.locations['images'], 'seen.png'))
    else:
        w['seen_icon'].set_from_file(os.path.join(self.locations['images'], 'unseen.png'))
    if 'notes' in item and item['notes']:
        w['notes'].set_text(str(item.notes))
    else:
        w['notes'].set_text('')
    tmp = ''
    if 'media_num' in item and item['media_num']:
        tmp = str(item.media_num)
    else:
        tmp = '0'
    if 'medium_id' in item and item['medium_id']:
        if item.medium is not None:
            tmp += ' x ' + item.medium.name
        else:
            pass
    w['medium'].set_markup("<i>%s</i>" % gutils.html_encode(tmp))
    if 'vcodec_id' in item:
        if item.vcodec is not None:
            w['vcodec'].set_markup("<i>%s</i>" % gutils.html_encode(item.vcodec.name))
        else:
            w['vcodec'].set_text('')
    else:
        w['vcodec'].set_text('')
    # TODO: add widgets in main and edit windows
    #if 'ratio_id' in item:
    #    if item.ratio is not None:
    #        w['ratio'].set_markup("<i>%s</i>" % gutils.html_encode(item.ratio.name))
    #    else:
    #        w['ratio'].set_text('')
    #else:
    #    w['ratio'].set_text('')

    # poster
    if 'poster_md5' in item and item['poster_md5']:
        filename = gutils.get_image_fname(item['poster_md5'], self.db, 'm')
        if filename and os.path.isfile(filename):
            image_path = filename
            self.widgets['add']['delete_poster'].set_sensitive(True)
            w['picture_button'].set_sensitive(True)
        else:
            image_path = gutils.get_defaultimage_fname(self)
            self.widgets['add']['delete_poster'].set_sensitive(False)
            w['picture_button'].set_sensitive(False)
    else:
        image_path = gutils.get_defaultimage_fname(self)
        self.widgets['add']['delete_poster'].set_sensitive(False)
        w['picture_button'].set_sensitive(False)
    w['picture'].set_from_file(image_path)
    # ratig
    rimage = int(self.config.get('rating_image', 0))
    if rimage:
        prefix = ''
    else:
        prefix = 'meter'
    if 'rating' in item and item['rating']:
        rating_file = "%s/%s0%d.png" % (self.locations['images'], prefix, item['rating'])
    else:
        rating_file = "%s/%s0%d.png" % (self.locations['images'], prefix, 0)
    handler = w['image_rating'].set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(rating_file))
    gutils.garbage(handler)

    # check loan status and adjust buttons and history box
    if 'loaned' in item and item['loaned'] is True:
        self.widgets['popups']['loan'].set_sensitive(False)
        self.widgets['popups']['email'].set_sensitive(True)
        self.widgets['popups']['return'].set_sensitive(True)
        self.widgets['menu']['loan'].set_sensitive(False)
        self.widgets['menu']['email'].set_sensitive(True)
        self.widgets['menu']['return'].set_sensitive(True)
        w['loan_button'].set_sensitive(False)
        w['email_reminder_button'].set_sensitive(True)
        w['return_button'].set_sensitive(True)

        if getattr(item, 'loan_details', None) is None:
            log.warning("movie has no loan data, changing 'loaned' flag to False (movie_id: %s)", item['movie_id'])
            item.loaned = False
        else:
            self.person_name = item.loan_details.person.name
            self.person_email = item.loan_details.person.email
            self.loan_date = str(item.loan_details.date)
            w['loan_info'].set_use_markup(False)
            w['loan_info'].set_label(_("This movie has been loaned to %s on %s") % (self.person_name, self.loan_date[:10]))
    if 'loaned' in item and not item['loaned']: # "loaned" status can be changed above, so don't use "else:" in this line
        self.widgets['popups']['loan'].set_sensitive(True)
        self.widgets['popups']['email'].set_sensitive(False)
        self.widgets['popups']['return'].set_sensitive(False)
        self.widgets['menu']['loan'].set_sensitive(True)
        self.widgets['menu']['email'].set_sensitive(False)
        self.widgets['menu']['return'].set_sensitive(False)
        w['return_button'].set_sensitive(False)
        w['email_reminder_button'].set_sensitive(False)
        w['loan_button'].set_sensitive(True)
        w['loan_info'].set_markup("<b>%s</b>" % _("Movie not loaned"))

    # loan history
    self.loans_treemodel.clear()
    if getattr(item, 'loan_history', None) is not None:
        for loan in item.loan_history:
            myiter = self.loans_treemodel.append(None)
            self.loans_treemodel.set_value(myiter, 0, '%s' % str(loan.date)[:10])
            if loan.return_date and  loan.return_date != '':
                self.loans_treemodel.set_value(myiter, 1, str(loan.return_date)[:10])
            else:
                self.loans_treemodel.set_value(myiter, 1, "---")
            if loan.person:
                person = self.db.session.query(db.Person.name).filter_by(person_id=loan.person.person_id).first()
                self.loans_treemodel.set_value(myiter, 2, person.name)

    # volumes/collections
    if 'volume_id' in item and item['volume_id'] > 0:
        if 'volume' in item and item['volume']:
            w['volume'].set_markup("<b>%s</b>" % gutils.html_encode(item['volume'].name))
            w['show_volume_button'].set_sensitive(True)
        else:
            w['volume'].set_text('')
            w['show_volume_button'].set_sensitive(False)
    else:
            w['volume'].set_text('')
            w['show_volume_button'].set_sensitive(False)
    if 'collection_id' in item and item['collection_id'] > 0:
        if 'collection' in item and item['collection']:
            w['collection'].set_markup("<b>%s</b>" % gutils.html_encode(item['collection'].name))
            w['show_collection_button'].set_sensitive(True)
        else:
            w['collection'].set_text('')
            w['show_collection_button'].set_sensitive(False)
    else:
        w['collection'].set_text('')
        w['show_collection_button'].set_sensitive(False)

    # languages
    for i in w['audio_vbox'].get_children():
        i.destroy()
    for i in w['subtitle_vbox'].get_children():
        i.destroy()
    if 'languages' in item and len(item['languages']) > 0:
        for i in item['languages']:
            if i.type == 3: # subtitles
                if i.subformat:
                    tmp = "%s - %s" % (i.language.name, i.subformat.name)
                else:
                    tmp = "%s" % i.language.name
                w['subtitle_vbox'].pack_start(gtk.Label(tmp))
            else:
                language = i.language.name
                if i.type is not None and len(self._lang_types[i.type]) > 0:
                    language += " <i>%s</i>" % self._lang_types[i.type]
                tmp = ''
                if i.achannel:
                    tmp = i.achannel.name
                if i.acodec:
                    if len(tmp) > 0:
                        tmp += ", %s" % i.acodec.name
                    else:
                        tmp = i.acodec.name
                if len(tmp) > 0:
                    tmp = "%s (%s)" % (language, tmp)
                else:
                    tmp = language
                widget = gtk.Label(tmp)
                widget.set_use_markup(True)
                w['audio_vbox'].pack_start(widget)
    w['audio_vbox'].show_all()
    w['subtitle_vbox'].show_all()
    #tags
    if 'tags' in item:
        tmp = ''
        for tag in item['tags']:
            tmp += "%s, " % tag.name
        tmp = tmp[:-2] # cut last comma
        w['tags'].set_text(tmp)
    #}}}


def populate(self, movies=None, where=None, qf=True):#{{{
    if self.initialized is False: # dont try to fill movie list if Griffith is not initialized yet
        return False

    if qf and movies is None or isinstance(movies, Select): # if ".execute().fetchall()" not invoked on movies yet
        if not where: # due to possible 'seen', 'loaned', 'collection_id' in where
            import advfilter

            # saved in advfilter
            name = self.widgets['filter']['advfilter'].get_active_text()[:-3].decode('utf-8') # :-3 due to additional '   ' in the name
            if name:
                cond = self.db.session.query(db.Filter).filter_by(name=name).first()
                if not cond:
                    cond = advfilter.get_def_conditions()
                else:
                    cond = cond.data
            else:
                cond = advfilter.get_def_conditions()
            # add sorting from config
            sort_column_name = self.config.get('sortby', 'number', section='mainlist')
            sort_reverse = self.config.get('sortby_reverse', False, section='mainlist')
            if sort_reverse:
                cond['sort_by'] = set((sort_column_name + ' DESC', ))
            else:
                cond['sort_by'] = set((sort_column_name, ))

            # seen / loaned
            if self.widgets['menu']['loaned_movies'].get_active():
                cond['loaned'] = True
            if self.widgets['menu']['not_seen_movies'].get_active():
                cond["seen"] = False
            # collection
            pos = self.widgets['filter']['collection'].get_active()
            if pos >= 0:
                col_id = self.collection_combo_ids[pos]
                if col_id > 0:
                    cond["collections"].add(col_id)

            movies = advfilter.create_select_query(self, None, cond, movies)
        else:
            # select sort column
            sort_column_name = self.config.get('sortby', 'number', section='mainlist')
            sort_reverse = self.config.get('sortby_reverse', False, section='mainlist')
            # explicit conditions, only empty dictionary needed to add the order by values
            cond = {}
            if sort_reverse:
                cond['sort_by'] = set((sort_column_name + ' DESC', ))
            else:
                cond['sort_by'] = set((sort_column_name, ))
            movies = sql.update_whereclause(movies, cond)

        # additional whereclause (volume_id, collection_id, ...)
        if where:
            for i in where:
                if i in db.Movie:
                    movies.append_whereclause(db.Movie[i] == where[i])
        movies = movies.execute().fetchall()

    self.total = len(movies)
    # disable refreshing while inserting
    self.widgets['treeview'].freeze_child_notify()
    self.widgets['treeview'].set_model(None)

    # save user sort column
    sort_column_id, order = self.treemodel.get_sort_column_id()

    # new treemodel (faster and prevents some problems)
    self.treemodel = gtk.TreeStore(str, gtk.gdk.Pixbuf, str, str, str, str, bool, str, str, int, str, str)

    # check preferences to hide or show columns
    if self.config.get('number', True, 'mainlist') == True:
        self.number_column.set_visible(True)
    else:
        self.number_column.set_visible(False)
    if self.config.get('otitle', True, 'mainlist') == True:
        self.otitle_column.set_visible(True)
    else:
        self.otitle_column.set_visible(False)
    if self.config.get('title', True, 'mainlist') == True:
        self.title_column.set_visible(True)
    else:
        self.title_column.set_visible(False)
    if self.config.get('director', True, 'mainlist') == True:
        self.director_column.set_visible(True)
    else:
        self.director_column.set_visible(False)
    if self.config.get('image', True, 'mainlist') == True:
        self.image_column.set_visible(True)
    else:
        self.image_column.set_visible(False)
    if self.config.get('genre', True, 'mainlist') == True:
        self.genre_column.set_visible(True)
    else:
        self.genre_column.set_visible(False)
    if self.config.get('seen', True, 'mainlist') == True:
        self.seen_column.set_visible(True)
    else:
        self.seen_column.set_visible(False)
    if self.config.get('year', True, 'mainlist') == True:
        self.year_column.set_visible(True)
    else:
        self.year_column.set_visible(False)
    if self.config.get('runtime', True, 'mainlist') == True:
        self.runtime_column.set_visible(True)
    else:
        self.runtime_column.set_visible(False)
    if self.config.get('rating', True, 'mainlist') == True:
        self.rating_column.set_visible(True)
    else:
        self.rating_column.set_visible(False)
    if self.config.get('created', True, 'mainlist') == True:
        self.created_column.set_visible(True)
    else:
        self.created_column.set_visible(False)
    if self.config.get('updated', True, 'mainlist') == True:
        self.updated_column.set_visible(True)
    else:
        self.updated_column.set_visible(False)

    for movie in movies:
        addmovie(self, movie)

    # restore user sort column
    if sort_column_id is not None:
        self.treemodel.set_sort_column_id(sort_column_id, order)

    # add new treemodel and allow refreshs again
    self.widgets['treeview'].set_model(self.treemodel)
    self.widgets['treeview'].thaw_child_notify()
    if self.total:
        self.widgets['treeview'].set_cursor_on_cell(0)
    self.count_statusbar()
#}}}

def addmovie(self, movie): #{{{
    myiter = self.treemodel.append(None)
    setmovie(self, movie, myiter)
    return myiter
#}}}

def setmovie(self, movie, iter, treemodel = None): #{{{
    if not treemodel:
        treemodel = self.treemodel

    treemodel.set_value(iter, 0, '%004d' % int(movie.number))

    if self.config.get('image', True, section='mainlist') == True:
        filename = None
        if movie.poster_md5:
            filename = gutils.get_image_fname(movie.poster_md5, self.db, "s")
        if not filename:
            filename = os.path.join(self.locations['images'], 'default_thumbnail.png')

        self.Image.set_from_file(filename)
        try:
            pixbuf = self.Image.get_pixbuf()
            treemodel.set_value(iter, 1, pixbuf)
        except:
            log.exception("can't load the image %s" % filename)

    treemodel.set_value(iter, 2, movie.o_title)
    treemodel.set_value(iter, 3, movie.title)
    treemodel.set_value(iter, 4, movie.director)
    treemodel.set_value(iter, 5, movie.genre)
    treemodel.set_value(iter, 6, movie.seen)
    if movie.year is not None and (isinstance(movie.year, int) or isinstance(movie.year, long)):
        treemodel.set_value(iter, 7, str(movie.year))
    if movie.runtime is not None and (isinstance(movie.runtime, int) or isinstance(movie.runtime, long)):
        treemodel.set_value(iter, 8, '%003d' % movie.runtime + _(' min'))
    if movie.rating is not None and (isinstance(movie.rating, int) or isinstance(movie.rating, long)):
        treemodel.set_value(iter, 9, movie.rating)
    if movie.created:
        treemodel.set_value(iter, 10, movie.created.strftime('%Y-%m-%d %H:%M'))
    if movie.updated:
        treemodel.set_value(iter, 11, movie.updated.strftime('%Y-%m-%d %H:%M'))
    return iter
#}}}

def select(self, iter, ensurevisible = True): #{{{
    if iter:
        self.widgets['treeview'].get_selection().select_iter(iter)
        if ensurevisible:
            self.widgets['treeview'].scroll_to_cell(self.treemodel.get_path(iter))
    else:
        self.widgets['treeview'].get_selection().unselect_all()
    self.treeview_clicked()
#}}}

# vim: fdm=marker



# -*- coding: utf-8 -*-

__revision__ = '$Id: PluginMovieStopklatka.py 1383 2010-01-05 21:04:00Z mikej06 $'

# Copyright (c) 2005-2010 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string
import re

plugin_name         = 'Stopklatka'
plugin_description  = 'Internetowy Serwis Filmowy'
plugin_url          = 'www.stopklatka.pl'
plugin_language     = _('Polish')
plugin_author       = 'Piotr Ożarowski, Bartosz Kurczewski'
plugin_author_email = '<bartosz.kurczewski@gmail.com>'
plugin_version      = '1.11'


class Plugin(movie.Movie):
    IMAGE_PATTERN = re.compile('(http://img.stopklatka.pl/film/.*?)"')

    def __init__(self, id):
        self.movie_id = id
        self.url = "http://www.stopklatka.pl/film/film.asp?fi=%s" % str(self.movie_id)
        self.encode = 'utf-8'

    def initialize(self):
        self.page = self.page.replace(u'\x9c', u'ś')
        self.page = self.page.replace(u'š', u'ą')
        self.res = re.findall("""<td class="middle_cell"><span class="bold">(.*?)</span>, (.*?), (.*?), (.*?) min</td>""", self.page)
        if len(self.res) == 0:
            self.res = [('', '', '', '')]
        url = "http://www.stopklatka.pl/film/film.asp?fi=%s&sekcja=osoby" % str(self.movie_id)
        self.creditspage = self.open_page(self.parent_window, url=url)
        self.creditspage = self.creditspage.replace(u'\x9c', u'ś')
        self.creditspage = self.creditspage.replace(u'š', u'ą')

    def get_image(self):
        image = self.IMAGE_PATTERN.findall(self.page)
        if len(image) and image[0] != 'http://img.stopklatka.pl/film/0.jpg':
            self.image_url = image[0]

    def get_o_title(self):
        self.o_title = gutils.trim(self.page, '<h2> (', ')</h2>')
        if self.o_title == '':
            self.o_title = self.get_title(True)

    def get_title(self, ret=False):
        data = gutils.trim(self.page, '<h1>', '</h1>')
        if ret is True:
            return data
        else:
            self.title = data

    def get_director(self):
        self.director = gutils.trim(self.creditspage, u'reżyseria: <', '</tr>')
        self.director = gutils.after(self.director, '>')
        self.director = self.director.replace('<br />', ', ')
        self.director = gutils.clean(self.director)
        if self.director.endswith(','):
            self.director = self.director[:-1]

    def get_plot(self):
        self.plot = gutils.trim(self.page, '<p>', '</p>')
        self.plot = gutils.before(self.plot, u'<b>Więcej ')

    def get_year(self):
        self.year = self.res[0][2]

    def get_runtime(self):
        self.runtime = self.res[0][3]

    def get_genre(self):
        self.genre = self.res[0][0]

    def get_cast(self):
        self.cast = gutils.trim(self.creditspage, 'obsada: <', '</td></tr>')
        self.cast = gutils.after(self.cast, '>')
        self.cast = self.cast.replace(' jako ', _(' as '))
        self.cast = string.replace(self.cast, '<br />', "\n")

    def get_classification(self):
        self.classification = ''

    def get_studio(self):
        self.studio = ""

    def get_o_site(self):
        self.o_site = gutils.trim(self.page, ">strona oficjalna:<", " target=_blank")
        self.o_site = gutils.after(self.o_site, "href=")

    def get_site(self):
        self.site = self.url

    def get_trailer(self):
        self.trailer = "http://www.stopklatka.pl/film/film.asp?fi=" + self.movie_id + "&sekcja=mmedia"

    def get_country(self):
        self.country = self.res[0][1]

    def get_rating(self):
        self.rating = 0
        tmp = re.search(u'Ocena użytkowników:[^(]+[(]([0-9]+[,]*[0-9]*)[)]', self.page)
        if tmp:
            try:
                self.rating = int(round(float(tmp.group(1).replace(',', '.')), 0))
            except:
                pass

    def get_notes(self):
        self.notes = ''

    def get_cameraman(self):
        self.cameraman = gutils.regextrim(self.creditspage, u'zdjęcia: <', '(</tr>|<tr>)')
        self.cameraman = gutils.after(self.cameraman, '>')
        self.cameraman = self.cameraman.replace('<br />', ', ')
        self.cameraman = gutils.clean(self.cameraman)
        if self.cameraman.endswith(','):
            self.cameraman = self.cameraman[:-1]

    def get_screenplay(self):
        self.screenplay = gutils.trim(self.creditspage, u'scenariusz: <', '</tr>')
        self.screenplay = gutils.after(self.screenplay, '>')
        self.screenplay = self.screenplay.replace('<br />', ', ')
        self.screenplay = gutils.clean(self.screenplay)
        if self.screenplay.endswith(','):
            self.screenplay = self.screenplay[:-1]


class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        self.encode = 'utf-8'
        self.original_url_search = "http://www.stopklatka.pl/szukaj/szukaj.asp?kategoria=film&szukaj="
        self.translated_url_search = "http://www.stopklatka.pl/szukaj/szukaj.asp?kategoria=film&szukaj="

    def search(self, parent_window):
        if not self.open_search(parent_window):
            return None
        self.page = gutils.trim(self.page, '>Wyniki poszukiwania frazy:', '</div>')
        self.page = self.page.replace(u'\x9c', u'ś')
        self.page = self.page.replace(u'š', u'ą')
        return self.page

    def get_searches(self):
        elements = re.findall("""/film/film.asp\?fi=(\d+)"[^>]*>.*?searchTitle\s*textB">(.*?)</span>.*?"> (.*?)</span>""", self.page)
        self.number_results = len(elements)

        for element in elements:
            self.ids.append(element[0])
            self.titles.append(gutils.clean(element[1]) + ' ' + element[2])



# -*- coding: iso-8859-15 -*-

__revision__ = '$Id: PluginMovieFilmAffinity.py 389 2006-07-29 18:43:35Z piotrek $'

# Copyright (c) 2006-2011 Pedro D. Sánchez
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string, re

plugin_name         = 'FilmAffinity'
plugin_description  = 'Base de Datos de Peliculas'
plugin_url          = 'www.filmaffinity.com'
plugin_language     = _('Spanish')
plugin_author       = 'Pedro D. Sánchez'
plugin_author_email = '<pedrodav@gmail.com>'
plugin_version      = '0.6'

class Plugin(movie.Movie):
    def __init__(self, id):
        self.encode   = 'iso-8859-1'
        self.movie_id = id
        self.url      = "http://www.filmaffinity.com/es/film%s.html" % str(self.movie_id)

    def initialize(self):
        self.page = self.page.replace(u'\x91', '\'')
        self.page = self.page.replace(u'\x92', '\'')

    def get_image(self):
        tmp = re.search('pics[0-9]*.filmaffinity.com/', self.page)
        if not tmp:
            self.image_url = ''
        else:
            self.image_url = 'http://' + gutils.before(self.page[tmp.start():], '"')

    def get_o_title(self):
        self.o_title = gutils.trim(self.page, u'<th>T&Iacute;TULO ORIGINAL</th>', '</strong></td>')
        self.o_title = gutils.after(self.o_title, '<strong>')
        self.o_title = re.sub('[ ]+', ' ', self.o_title)
        self.o_title = re.sub('([(]Serie de TV[)]|[(]TV[)]|[(]TV Series[)])', '', self.o_title)

    def get_title(self):
        self.title = gutils.trim(self.page, 'www.filmaffinity.com/images/movie.gif" border="0"> ', '</span>')
        self.title = re.sub('[ ]+', ' ', self.title)
        self.title = re.sub('([(]Serie de TV[)]|[(]TV[)]|[(]TV Series[)])', '', self.title)

    def get_director(self):
        self.director = gutils.trim(self.page,'<th>DIRECTOR</th>', '</td>')

    def get_plot(self):
        self.plot = gutils.trim(self.page, '<th>SINOPSIS</th>', '</td>')
        self.plot = string.replace(self.plot, ' (FILMAFFINITY)', '')
        self.plot = string.replace(self.plot, '(FILMAFFINITY)', '')

    def get_year(self):
        self.year = gutils.trim(self.page, '<th>A&Ntilde;O</th>', '</td>')
        self.year = gutils.clean(self.year)

    def get_runtime(self):
        self.runtime = gutils.trim(self.page, '<th>DURACI&Oacute;N</th>', ' min.')
        self.runtime = gutils.after(self.runtime[-10:], '<td>')

    def get_genre(self):
        self.genre = ''
        tmp = gutils.trim(self.page, '<th>G&Eacute;NERO</th>', '</tr>')
        tmp = gutils.after(tmp, '<td>')
        if tmp:
            self.genre = gutils.clean(string.replace(tmp, ' | ', '. '))
            self.genre = re.sub('[.][ \t]+', '. ', self.genre)

    def get_cast(self):
        self.cast = ''
        self.cast = gutils.trim(self.page, '<th>REPARTO</th>', '</td>')
        self.cast = re.sub('</a>,[ ]*', '\n', self.cast)
        self.cast = string.strip(gutils.strip_tags(self.cast))
        self.cast = re.sub('[ ]+', ' ', self.cast)
        self.cast = re.sub('\n[ ]+', '\n', self.cast)

    def get_classification(self):
        self.classification = ''

    def get_studio(self):
        self.studio = gutils.trim(self.page, '<th>PRODUCTORA</th>', '</td>')
        self.studio = gutils.after(self.studio, '<td>')

    def get_o_site(self):
        self.o_site = gutils.trim(self.page, '<th>WEB OFICIAL</th>', '</a>')
        self.o_site = gutils.after(self.o_site, '">')

    def get_site(self):
        self.site = "http://www.filmaffinity.com/es/film%s.html" % str(self.movie_id)

    def get_trailer(self):
        self.trailer = ''

    def get_country(self):
        self.country = gutils.trim(self.page, '<b>PA&Iacute;S</th>', '</td>')
        tmp = gutils.trim(self.country, 'alt="', '"')
        if tmp == '':
            self.country = gutils.trim(self.country, 'title="', '"')
        else:
            self.country = tmp

    def get_rating(self):
        self.rating = gutils.trim(self.page, '<div style="margin: 4px 0; color:#990000; font-size:22px; font-weight: bold;">', '</div>')
        if self.rating:
            self.rating = str(round(float(gutils.clean(string.replace(self.rating, ',', '.')))))

    def get_cameraman(self):
        self.cameraman = gutils.trim(self.page, '<th>FOTOGRAF&Iacute;A</th>','</td>')

    def get_screenplay(self):
        self.screenplay = gutils.trim(self.page, '<th>GUI&Oacute;N</th>', '</td>')

class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        self.original_url_search   = 'http://www.filmaffinity.com/es/search.php?stype=title&stext='
        self.translated_url_search = 'http://www.filmaffinity.com/es/search.php?stype=title&stext='
        self.encode                = 'iso-8859-1'

    def search(self, parent_window):
        if not self.open_search(parent_window):
            return None
        auxPage = self.page
        self.sub_search(parent_window)
        if self.page <> '':
            return self.page
        auxPage = gutils.trim(auxPage, u'<b>TU CRÍTICA</b></div>', '</a>')
        self.page = gutils.trim(auxPage, 'movie_id=', '">')
        return self.page

    def sub_search(self, parent_window):
        moviepage = gutils.trim(self.page, u'Resultados por título</span>', '<div id="bpanel">')
        nextpage = self.get_nextpage_url()
        while nextpage:
            self.url = nextpage
            self.open_search(parent_window)
            moviepage = moviepage + gutils.trim(self.page, u'Resultados por título</span>', '<div id="bpanel">')
            nextpage = self.get_nextpage_url()
        self.page = moviepage

    def get_nextpage_url(self):
        match = re.search('(siguientes >|siguientes &gt;)', self.page)
        if match:
            start = string.rfind(self.page, '<a href="', 0, match.start())
            if start >= 0:
                return 'http://www.filmaffinity.com/es/' + gutils.before(self.page[start + 9:match.start()], '"')
        return None

    def get_searches(self):
        if not self.page:
            return
        if len(self.page) < 20:    # immidietly redirection to movie page
            self.number_results = 1
            self.ids.append(self.page)
            self.titles.append(self.url)
        else:            # multiple matches
            elements = string.split(self.page, '</a></b>')

            if (elements[0]<>''):
                for index in range(0, len(elements) - 1, 1):
                    element = elements[index]
                    nextelement = elements[index + 1]
                    id = gutils.trim(element, '<b><a href="/es/film', '.html')
                    if id:
                        self.ids.append(id)
                        title = gutils.clean(gutils.after(element, '<b><a href="/es/film')).replace("\n", "")
                        title = gutils.strip_tags(gutils.convert_entities(gutils.after(title, '>'))) + ' ' + string.strip(gutils.before(nextelement, '<'))
                        self.titles.append(title)

#
# Plugin Test
#
class SearchPluginTest(SearchPlugin):
    #
    # Configuration for automated tests:
    # dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
    #
    test_configuration = {
        'Rocky' : [ 17, 17 ],
        'Darkness' : [59, 59 ]
    }

class PluginTest:
    #
    # Configuration for automated tests:
    # dict { movie_id -> dict { arribute -> value } }
    #
    # value: * True/False if attribute only should be tested for any value
    #        * or the expected value
    #
    test_configuration = {
        '706925' : {
            'title'               : 'Rocky Balboa (Rocky VI)',
            'o_title'             : 'Rocky Balboa (Rocky VI)',
            'director'            : 'Sylvester Stallone',
            'plot'                : True,
            'cast'                : 'Sylvester Stallone\n\
Burt Young\n\
Antonio Tarver\n\
Geraldine Hughes\n\
Milo Ventimiglia\n\
Tony Burton\n\
James Francis Kelly III\n\
Talia Shire\n\
Angela Boyd\n\
A.J. Benza\n\
Mike Tyson',
            'country'             : 'Estados Unidos',
            'genre'               : u'Acción. Drama. Deporte. Boxeo. Secuela',
            'classification'      : False,
            'studio'              : 'MGM / UA / Columbia Pictures / Revolution Studios',
            'o_site'              : 'http://www.rockythemovie.com',
            'site'                : 'http://www.filmaffinity.com/es/film706925.html',
            'trailer'             : False,
            'year'                : 2006,
            'notes'               : False,
            'runtime'             : 102,
            'image'               : True,
            'rating'              : 6,
            'cameraman'           : 'J. Clark Mathis',
            'screenplay'          : 'Sylvester Stallone (Personajes: Sylvester Stallone)',
            'barcode'             : False
        },
        '373499' : {
            'title'               : 'Camino',
            'o_title'             : 'Camino',
            'director'            : 'Javier Fesser',
            'plot'                : True,
            'cast'                : u'Nerea Camacho\n\
Carmen Elías\n\
Mariano Venancio\n\
Manuela Vellés\n\
Ana Gracia\n\
Lola Casamayor\n\
Lucas Manzano\n\
Pepe Ocio\n\
Claudia Otero\n\
Jordi Dauder\n\
Emilio Gavira\n\
Miriam Raya\n\
Jan Cornet',
            'country'             : u'España',
            'genre'               : u'Drama. Enfermedad. Religión. Basado en hechos reales',
            'classification'      : False,
            'studio'              : u'Películas Pendelton / Mediapro',
            'o_site'              : 'http://www.caminolapelicula.com/',
            'site'                : 'http://www.filmaffinity.com/es/film373499.html',
            'trailer'             : False,
            'year'                : 2008,
            'notes'               : False,
            'runtime'             : 143,
            'image'               : True,
            'rating'              : 7,
            'cameraman'           : u'Alex Catalán',
            'screenplay'          : 'Javier Fesser',
            'barcode'             : False
        },
        '580875' : {
            'title'               : 'El gato negro (Masters of Horror Series)',
            'o_title'             : 'Black Cat (Masters of Horror Series), The',
            'director'            : 'Stuart Gordon',
            'plot'                : True,
            'cast'                : 'Jeffrey Combs\n\
Patrick Gallagher\n\
Eric Keenleyside\n\
Christopher Heyerdahl',
            'country'             : 'Estados Unidos',
            'genre'               : u'Terror. Biográfico. Drama psicológico. Siglo XIX. Serie [Masters of Horror]',
            'classification'      : False,
            'studio'              : 'Emitida por la cadena Showtime',
            'o_site'              : False,
            'site'                : 'http://www.filmaffinity.com/es/film580875.html',
            'trailer'             : False,
            'year'                : 2007,
            'notes'               : False,
            'runtime'             : 59,
            'image'               : True,
            'rating'              : 6,
            'cameraman'           : 'David Pelletier',
            'screenplay'          : 'Dennis Paoli, Stuart Gordon (Historia corta: Edgar Allan Poe)',
            'barcode'             : False
        },
        '826399' : {
            'title'               : 'Surcadores del cielo (The Sky Crawlers)',
            'o_title'             : 'Sukai kurora (The Sky Crawlers)',
            'director'            : 'Mamoru Oshii',
            'plot'                : True,
            'cast'                : 'Animation',
            'country'             : u'Japón',
            'genre'               : u'Animación. Aventuras. Bélico',
            'classification'      : False,
            'studio'              : 'Production I.G. / Nippon Television Network Corporation (NTV)',
            'o_site'              : 'http://sky.crawlers.jp/',
            'site'                : 'http://www.filmaffinity.com/es/film826399.html',
            'trailer'             : False,
            'year'                : 2008,
            'notes'               : False,
            'runtime'             : 122,
            'image'               : True,
            'rating'              : 6,
            'cameraman'           : 'Animation',
            'screenplay'          : 'Chihiro Itou (Novela: Hiroshi Mori)',
            'barcode'             : False
        },
    }



# -*- coding: UTF-8 -*-

__revision__ = '$Id: macutils.py 1519 2011-02-05 15:32:36Z iznogoud $'

# Copyright (c) 2005-2011 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from Cocoa import *

# ================
# Save file dialog
# ================

def saveDialog(filetypes=[]):
    panel = NSSavePanel.savePanel()
    panel.setCanCreateDirectories_(True)
    panel.setCanChooseDirectories_(False)
    panel.setCanChooseFiles_(True)
    panel.setAllowsOtherFileTypes_(False)
    result = panel.runModalForDirectory_file_types_(
        NSHomeDirectory() + '/Desktop',None,filetypes)
    if result:
        return True, panel.filename(), panel.directory()
    else:
        return False, None, None
    return
    
# ================
# Open file dialog
# ================

def openDialog(filetypes=[]):
    panel = NSOpenPanel.openPanel()
    panel.setCanCreateDirectories_(False)
    panel.setCanChooseDirectories_(False)
    panel.setCanChooseFiles_(True)
    result = panel.runModalForDirectory_file_types_(
        NSHomeDirectory() + '/Desktop',None,filetypes)
    if result:
        return True, panel.filename(), panel.directory()
    else:
        return False, None, None
    return
    
# ================
# Alert dialog
# ================  
    
def createAlert(msg):
    alert = NSAlert.alloc().init()
    alert.setMessageText_(msg)
    alert.setInformativeText_("")
    alert.setAlertStyle_(NSInformationalAlertStyle)
    alert.runModal()
        
# ================
# Question dialog
# ================
        
def question(msg, window=None):
    alert = NSAlert.alertWithMessageText_defaultButton_alternateButton_otherButton_informativeTextWithFormat_(
        msg, "Yes", "No", None, "")
    buttonPressed=alert.runModal()
    if buttonPressed == 0:
        return False
    else:
        return True


# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovieOFDb.py 1599 2011-12-02 19:52:02Z mikej06 $'

# Written by Christian Sagmueller <christian@sagmueller.net>
# based on PluginMovieIMDB.py, Copyright (c) 2005 Vasco Nunes
# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie,string,re

plugin_name         = "OFDb"
plugin_description  = "Online-Filmdatenbank"
plugin_url          = "www.ofdb.de"
plugin_language     = _("German")
plugin_author       = "Christian Sagmueller, Jessica Katharina Parth"
plugin_author_email = "Jessica.K.P@women-at-work.org"
plugin_version      = "0.10"

class Plugin(movie.Movie):
    def __init__(self, id):
        self.encode   = 'utf-8'
        self.movie_id = id
        self.url      = "http://www.ofdb.de/%s" % str(self.movie_id)

    def initialize(self):
        # OFDb didn't provide the runtime, studio and classification but it provide a link to the german imdb entry
        # lets use the imdb page, why not
        imdb_nr = gutils.trim(self.page, 'http://german.imdb.com/Title?', '"')
        if imdb_nr != '':
            self.imdb_page = self.open_page(url='http://www.imdb.de/Title?' + imdb_nr)
        else:
            imdb_nr = gutils.trim(self.page, 'http://www.imdb.com/Title?', '"')
            if imdb_nr != '':
                self.imdb_page = self.open_page(url='http://www.imdb.de/Title?' + imdb_nr)
            else:
                self.imdb_page = ''

    def get_image(self):
        self.image_url = "http://img.ofdb.de/film/" + gutils.trim(self.page, 'img src="http://img.ofdb.de/film/', '"' )

    def get_o_title(self):
        self.o_title = gutils.clean(gutils.trim(self.page, 'Originaltitel:', '</tr>'))
        if self.o_title == '':
            self.o_title = string.replace(self.o_title, '&nbsp;', '' )

    def get_title(self):
        self.title = gutils.trim(self.page,'size="3"><b>','<')

    def get_director(self):
        self.director = gutils.trim(self.page,"Regie:","</a><br>")

    def get_plot(self):
        self.plot = ''
        storyid = gutils.regextrim(self.page, '<a href="plot/', '(">|[&])')
        if not storyid is None:
            story_page = self.open_page(url="http://www.ofdb.de/plot/%s" % (storyid.encode('utf8')))
            if story_page:
                self.plot = gutils.trim(story_page, "</b><br><br>","</")

    def get_year(self):
        self.year = gutils.trim(self.page,"Erscheinungsjahr:","</a>")
        self.year = gutils.strip_tags(self.year)

    def get_runtime(self):
        # from imdb
        self.runtime = gutils.regextrim(self.imdb_page, '<h5>(L&auml;nge|L&#xE4;nge|Runtime):</h5>', ' (min|Min)')

    def get_genre(self):
        self.genre = gutils.trim(self.page,"Genre(s):","</table>")
        self.genre = string.replace(self.genre, "<br>", ", ")
        self.genre = gutils.strip_tags(self.genre)
        self.genre = string.replace(self.genre, "/", ", ")
        self.genre = gutils.clean(self.genre)
        self.genre = self.genre[0:-1]

    def get_cast(self):
        self.cast = ''
        movie_id_elements = string.split(self.movie_id, ',')
        movie_id_elements[0] = string.replace(movie_id_elements[0], "film/", "")
        cast_page = self.open_page(url="http://www.ofdb.de/view.php?page=film_detail&fid=%s" % str(movie_id_elements[0]) )
        self.cast = gutils.trim(cast_page, 'Darsteller</i>', '</table>')
        self.cast = re.sub('(\n|\t|&nbsp;)', '', self.cast)
        self.cast = string.replace(self.cast, '\t', '')
        self.cast = string.replace(self.cast, 'class="Daten">', '>\n')
        self.cast = string.strip(gutils.strip_tags(self.cast))
        self.cast = string.replace(self.cast, '... ', _(' as '))
        self.cast = gutils.clean(self.cast)

    def get_classification(self):
        # from imdb
        self.classification = gutils.regextrim(gutils.regextrim(self.imdb_page, '(Altersfreigabe|Certification):', '</div>'), '(Deutschland|Germany):', '(&|[|])')

    def get_studio(self):
        # from imdb
        self.studio = gutils.regextrim(self.imdb_page, '<h5>(Firma|Company):</h5>', '</a>')

    def get_o_site(self):
        self.o_site = ""

    def get_site(self):
        self.site = self.url

    def get_trailer(self):
        self.trailer = ""

    def get_country(self):
        self.country = gutils.trim(self.page,"Herstellungsland:","</a>")

    def get_rating(self):
        self.rating = gutils.trim(self.page,"<br>Note: ","&nbsp;")
        if self.rating == '':
            self.rating = "0"
        self.rating = str(round(float(self.rating)))

class SearchPlugin(movie.SearchMovie):
    def __init__(self):
        self.original_url_search   = "http://www.ofdb.de/view.php?page=suchergebnis&Kat=OTitel&SText="
        self.translated_url_search = "http://www.ofdb.de/view.php?page=suchergebnis&Kat=DTitel&SText="
        self.encode                = 'utf-8'
        self.remove_accents        = False

    def search(self,parent_window):
        if not self.open_search(parent_window):
            return None
        self.page = gutils.trim(self.page,"</b><br><br>", "<br><br><br>");
        self.page = string.replace( self.page, "'", '"' )
        self.page = string.replace( self.page, '<font size="1">', '' )
        self.page = string.replace( self.page, '</font>', '' )
        return self.page

    def get_searches(self):
        elements = string.split(self.page,"<br>")

        if (elements[0]<>''):
            for element in elements:
                elementid = gutils.trim(element,'<a href="','"')
                if not elementid is None and not elementid == '':
                    self.ids.append(elementid)
                    elementname = gutils.clean(element)
                    p1 = string.find(elementname, '>')
                    if p1 == -1:
                        self.titles.append(elementname)
                    else:
                        self.titles.append(elementname[p1+1:])

#
# Plugin Test
#
class SearchPluginTest(SearchPlugin):
    #
    # Configuration for automated tests:
    # dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
    #
    test_configuration = {
        'Rocky Balboa' : [ 1, 1 ],
        'Arahan'       : [ 3, 2 ],
        'glückliches'  : [ 4, 2 ]
    }

class PluginTest:
    #
    # Configuration for automated tests:
    # dict { movie_id -> dict { arribute -> value } }
    #
    # value: * True/False if attribute only should be tested for any value
    #        * or the expected value
    #
    test_configuration = {
        'film/103013,Rocky%20Balboa' : {
            'title'               : 'Rocky Balboa',
            'o_title'             : 'Rocky Balboa',
            'director'            : 'Sylvester Stallone',
            'plot'                : True,
            'cast'                : 'Sylvester Stallone' + _(' as ') + 'Rocky Balboa\n\
Burt Young' + _(' as ') + 'Paulie\n\
Milo Ventimiglia' + _(' as ') + 'Robert Jr.\n\
Geraldine Hughes' + _(' as ') + 'Marie\n\
James Francis Kelly III' + _(' as ') + 'Steps\n\
Tony Burton' + _(' as ') + 'Duke\n\
A.J. Benza\n\
Henry G. Sanders' + _(' as ') + 'Martin\n\
Antonio Tarver' + _(' as ') + 'Mason \'The Line\' Dixon\n\
Pedro Lovell' + _(' as ') + 'Spider Rico\n\
Ana Gerena\n\
Angela Boyd\n\
Louis Giansante\n\
Maureen Schilling\n\
Carter Mitchell\n\
Vinod Kumar\n\
Tobias Segal' + _(' as ') + 'Robert\'s Friend\n\
Tim Carr' + _(' as ') + 'Robert\'s Friend \n\
Paul Dion Monte' + _(' as ') + 'Robert\'s Friend\n\
Kevin King Templeton\n\
Robert Michael Kelly\n\
Don Sherman' + _(' as ') + 'Andy\n\
Nick Baker\n\
Rick Buchborn\n\
Gary Compton\n\
Jody Giambelluca\n\
Frank Stallone als Frank Stallone Jr.' + _(' as ') + 'Dinner Patron \n\
Fran Pultro\n\
Michael Buffer' + _(' as ') + 'Ring Announcer\n\
Jack Lazzarado\n\
Marc Ratner\n\
Anthony Lato Jr.\n\
Yahya\n\
Gunnar Peterson\n\
Bernard Fernández\n\
Bert Randolph Sugar\n\
Jim Lampley\n\
Larry Merchant\n\
Max Kellerman\n\
James Binns\n\
Johnnie Hobbs Jr.\n\
Barney Fitzpatrick\n\
Brian Kenny\n\
Dana Jacobson\n\
Skip Bayless\n\
Charles Johnson\n\
Matt Frack\n\
Woody Paige\n\
Jay Crawford\n\
Lahmard J. Tate\n\
LeRoy Neiman\n\
Mike Tyson' + _(' as ') + 'Himself\n\
Lou DiBella\n\
Joe Cortez\n\
Ricky Cavazos' + _(' as ') + 'Boxing Spectator (uncredited)',
            'country'             : 'USA',
            'genre'               : 'Action, Drama, Sportfilm',
            'classification'      : False,
            'studio'              : 'Metro-Goldwyn-Mayer (MGM)',
            'o_site'              : False,
            'site'                : 'http://www.ofdb.de/film/103013,Rocky%20Balboa',
            'trailer'             : False,
            'year'                : 2006,
            'notes'               : False,
            'runtime'             : 102,
            'image'               : True,
            'rating'              : 8
        },
        'film/22489,Ein-Gl%C3%BCckliches-Jahr' : {
            'title'               : 'Glückliches Jahr, Ein',
            'o_title'             : 'Bonne année, La',
            'director'            : 'Claude Lelouch',
            'plot'                : False,
            'cast'                : 'Lino Ventura' + _(' as ') + 'Simon \n\
Françoise Fabian' + _(' as ') + 'Françoise\n\
Charles Gérard' + _(' as ') + 'Charlot\n\
André Falcon' + _(' as ') + 'Le bijoutier\n\
Mireille Mathieu\n\
Lilo\n\
Claude Mann\n\
Frédéric de Pasquale\n\
Gérard Sire\n\
Silvano Tranquilli' + _(' as ') + 'L\'amant italien \n\
André Barello\n\
Michel Bertay\n\
Norman de la Chesnaye\n\
Pierre Edeline\n\
Pierre Pontiche\n\
Michou\n\
Bettina Rheims\n\
Joseph Rythmann\n\
Georges Staquet\n\
Jacques Villedieu\n\
Harry Walter\n\
Elie Chouraqui',
            'country'             : 'Frankreich',
            'genre'               : 'Komödie, Krimi',
            'classification'      : '12 (f)',
            'studio'              : 'Les Films 13',
            'o_site'              : False,
            'site'                : 'http://www.ofdb.de/film/22489,Ein-Gl%C3%BCckliches-Jahr',
            'trailer'             : False,
            'year'                : 1973,
            'notes'               : False,
            'runtime'             : 90,
            'image'               : True,
            'rating'              : 6
        },
        'film/54088,Arahan' : {
            'title'               : 'Arahan',
            'o_title'             : 'Arahan jangpung daejakjeon',
            'director'            : 'Ryoo Seung-wan',
            'plot'                : True,
            'cast'                : 'Ryoo Seung-beom\n\
Yoon Soy' + _(' as ') + 'Wi-jin\n\
Ahn Seong-gi' + _(' as ') + 'Ja-woon\n\
Jeong Doo-hong' + _(' as ') + 'Heuk-Woon\n\
Yoon Joo-sang' + _(' as ') + 'Moo-woon \n\
Kim Ji-yeong\n\
Baek Chan-gi\n\
Kim Jae-man\n\
Lee Dae-yeon\n\
Kim Dong-ju\n\
Kim Su-hyeon\n\
Geum Dong-hyeon\n\
Lee Jae-goo\n\
Ahn Kil-kang\n\
Bong Tae-gyu' + _(' as ') + 'Cameo\n\
Im Ha-ryong' + _(' as ') + 'Cameo\n\
Yoon Do-hyeon\n\
Lee Choon-yeon' + _(' as ') + 'Cameo\n\
Kim Yeong-in\n\
Park Yoon-bae\n\
Lee Won\n\
Kim Kyeong-ae\n\
Yoo Soon-cheol\n\
Hwang Hyo-eun\n\
Lee Jae-ho\n\
Yang Ik-joon\n\
Kwon Beom-taek\n\
Min Hye-ryeong\n\
Oh Soon-tae\n\
Lee Oi-soo',
            'country'             : 'Südkorea',
            'genre'               : 'Action, Fantasy, Komödie',
            'classification'      : '16',
            'studio'              : 'Fun and Happiness',
            'o_site'              : False,
            'site'                : 'http://www.ofdb.de/film/54088,Arahan',
            'trailer'             : False,
            'year'                : 2004,
            'notes'               : False,
            'runtime'             : 114,
            'image'               : True,
            'rating'              : 7
        }
    }



# -*- coding: UTF-8 -*-
# vim: fdm=marker

__revision__ = '$Id: widgets.py 1572 2011-08-23 18:47:22Z mikej06 $'

# Copyright (c) 2005-2011 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gtk
import advfilter


def define_widgets(self, gladefile):
    get = lambda x: gladefile.get_widget(x)
    self.widgets = {}

    self.widgets['window'] = get('main_window')
    self.widgets['window'].connect('key_press_event', self.on_key_press_event)
    self.widgets['treeview'] = get('main_treeview')
    self.widgets['treeview'].connect('button_press_event', self.on_maintree_button_press_event)
    self.widgets['statusbar'] = get('statusbar')
    self.widgets['progressbar'] = get('w_progress')    # get from web
    #buttons
    self.widgets['new_db'] = get('new_bt')
    self.widgets['toolbar'] = get('toolbar1')


    self.widgets['movie'] = {#{{{
        'cast': get('m_cast'),
        'classification': get('m_classification'),
        'cameraman': get('m_cameraman'),
        'collection': get('m_collection'),
        'color': get('m_color'),
        'condition': get('m_condition'),
        'country': get('m_country'),
        'director': get('m_director'),
        'genre': get('m_genre'),
        'imdb': get('m_imdb'),
        'layers': get('m_layers'),
        'loan_info': get('loan_information'),
        'medium': get('m_medium'),
        'notes': get('m_notes'),
        'number': get('m_number'),
        'o_title': get('m_o_title'),
        'picture': get('m_picture_image'),
        'picture_button': get('m_picture'),
        'plot': get('m_plot'),
        'region': get('m_region'),
        'resolution': get('m_resolution'),
        'runtime': get('m_runtime'),
        'screenplay': get('m_screenplay'),
        'seen_icon': get('m_seen_icon'),
        'site': get('m_site'),
        'studio': get('m_studio'),
        'ratio': get('m_ratio'),
        'tags': get('m_tags'),
        'title': get('m_title'),
        'trailer': get('m_trailer'),
        'vcodec': get('m_vcodec'),
        'volume': get('m_volume'),
        'year': get('m_year'),
        'audio_vbox': get('m_audio_vbox'),
        'subtitle_vbox': get('m_subtitle_vbox'),
        'show_collection_button': get('m_show_collection_button'),
        'show_volume_button': get('m_show_volume_button'),
        'go_o_site_button': get('go_o_site'),
        'go_site_button': get('go_site'),
        'go_trailer_button': get('go_trailer'),
        'return_button': get('return_button'),
        'loan_button': get('loan_button'),
        'loan_history': get('loan_history'),
        'loan_to': get('loan_to'),
        'email_reminder_button': get('b_email_reminder'),
        'image_rating': get('m_image_rating'),
    }
    get('m_seen_icon_eventbox').connect('button_press_event', self.on_m_seen_icon_button_press_event)
    #}}}

    self.widgets['add'] = {#{{{
        'window': get('add_movie'),
        'notebook': get('notebook_add'),
        'barcode': get('am_barcode'),
        'classification': get('am_classification'),
        'cameraman': get('am_cameraman'),
        'collection': get('am_collection_combo'),
        'color': get('am_color'),
        'condition': get('am_condition'),
        'country': get('am_country'),
        'director': get('am_director'),
        'discs': get('am_discs'),
        'genre': get('am_genre'),
        'site': get('am_imdb'),
        'layers': get('am_layers'),
        'media': get('am_media'),
        'medium_vbox': get('am_medium_vbox'),
        'number': get('am_number'),
        'notes': get('am_obs'),
        'o_title': get('am_original_title'),
        'picture': get('am_image'),
        'image': get('am_hide_image_name'),
        'plot': get('am_plot'),
        'plugin_desc': get('am_plugin_desc'),
        'plugin_image': get('am_plugin_image'),
        #'ratio': get('am_ratio'), # TODO
        'region': get('am_region'),
        'resolution': get('am_resolution'),
        'runtime': get('am_runtime'),
        'seen': get('am_seen'),
        'o_site': get('am_site'),
        'screenplay': get('am_screenplay'),
        'source': get('am_source'),
        'studio': get('am_studio'),
        'tag_vbox': get('am_tag_vbox'),
        'title': get('am_title'),
        'trailer': get('am_trailer'),
        'vcodec': get('am_vcodec'),
        'volume': get('am_volume_combo'),
        'cast': get('am_with'),
        'year': get('am_year'),
        'image_rating': get('image_add_rating'),
        'rating_slider': get('rating_scale_add'),
        'lang_menu': get('lang_menu'),
        'lang_treeview': get('lang_treeview'),
        'b_get_from_web': get('get_from_web'),
        'c_web_source': get('combo_source'), # c_web_source
        'delete_poster': get('t_delete_poster'),
        'add_button': get('am_add_button'),
        'add_close_button': get('am_add_close_button'),
        'clear_button': get('am_clear_button'),
        'save_button': get('am_save_button'),
        'cb_only_empty': get('am_cb_only_empty'),
        'aremove_poster': get('aremove_poster'),
        'aadd_poster': get('aadd_poster'),
    }
    self.widgets['add']['window'].connect('delete_event', self.on_delete_event_am)
    self.widgets['add']['lang_treeview'].connect('button_press_event', self.on_lang_treeview_button_press_event)
    self.widgets['add']['o_title'].connect('activate', self.on_enter)
    self.widgets['add']['title'].connect('activate', self.on_enter)
    self.widgets['add']['window'].set_transient_for(self.widgets['window'])
    self.widgets['add']['aadd_poster'].connect('button_press_event', self.a_change_poster)
    self.widgets['add']['aremove_poster'].connect('button_press_event', self.a_del_poster)

    #}}}

    self.widgets['advfilter'] = {#{{{
        'window': get('w_advfilter'),
        'advfilter_vbox': get('advfilter_rules_vbox'),
        'rb_seen': get('af_rb_seen'),
        'rb_seen_only': get('af_rb_seen_only'),
        'rb_seen_only_n': get('af_rb_seen_only_n'),
        'rb_loaned': get('af_rb_loaned'),
        'rb_loaned_only': get('af_rb_loaned_only'),
        'rb_loaned_only_n': get('af_rb_loaned_only_n'),
        'tags_vbox': get('af_tags_vbox'),
        'volumes_vbox': get('af_volumes_vbox'),
        'collections_vbox': get('af_collections_vbox'),
        'loans_vbox': get('af_loans_vbox'),
        'dynamic_vbox': get('af_dynamic_vbox'),
        'add_button': get('af_add_button'),
        'tags_frame': get('af_frame_tags'),
        'volumes_frame': get('af_frame_volumes'),
        'collections_frame': get('af_frame_collections'),
        'loans_frame': get('af_frame_loans'),
        'cb_name': get('af_cbe_name'),
        'name': get('af_cbe_name_entry'),
    }
    self.widgets['advfilter']['window'].connect('delete_event', self.hide_advfilter_window)
    self.widgets['advfilter']['window'].set_transient_for(self.widgets['window'])
    #}}}

    self.widgets['preferences'] = {#{{{
        'window': get('w_preferences'),
        'notebook': get('p_notebook'),
        'treeview': get('p_treeview'),
        'color': get('p_color'),
        'condition': get('p_condition'),
        'db_details': get('p_db_details'),
        'db_host': get('p_db_host'),
        'db_name': get('p_db_name'),
        'db_passwd': get('p_db_passwd'),
        'db_port': get('p_db_port'),
        'db_type': get('p_db_type'),
        'db_user': get('p_db_user'),
        'epdf_reader': get('pdf_reader_entry'),
        'font': get('p_font'),
        'font_size': get('p_font_size'),
        'pdf_elements_table': get('pdf_elements_table'),
        'pdf_image': get('pdf_image'),
        'pdf_director': get('pdf_director'),
        'pdf_genre': get('pdf_genre'),
        'pdf_cast': get('pdf_cast'),
        'pdf_plot': get('pdf_plot'),
        'pdf_year': get('pdf_year'),
        'pdf_runtime': get('pdf_runtime'),
        'pdf_notes': get('pdf_notes'),
        'layers': get('p_layers'),
        'media': get('p_media'),
        'region': get('p_region'),
        'seen': get('p_seen'),
        's_classification': get('p_s_classification'),
        's_country': get('p_s_country'),
        's_director': get('p_s_director'),
        's_genre': get('p_s_genre'),
        's_image': get('p_s_image'),
        's_notes': get('p_s_notes'),
        's_o_site': get('p_s_o_site'),
        's_o_title': get('p_s_o_title'),
        's_plot': get('p_s_plot'),
        's_rating': get('p_s_rating'),
        's_runtime': get('p_s_runtime'),
        's_site': get('p_s_site'),
        's_studio': get('p_s_studio'),
        's_title': get('p_s_title'),
        's_trailer': get('p_s_trailer'),
        's_cast': get('p_s_cast'),
        's_year': get('p_s_year'),
        's_screenplay': get('p_s_screenplay'),
        's_cameraman': get('p_s_cameraman'),
        's_resolution': get('p_s_resolution'),
        's_barcode': get('p_s_barcode'),
        's_limit': get('p_sp_s_limit'),
        'vcodec': get('p_vcodec'),
        'view_image': get('view_image'),
        'view_number': get('view_number'),
        'view_o_title': get('view_otitle'),
        'view_title': get('view_title'),
        'view_director': get('view_director'),
        'view_genre': get('view_genre'),
        'view_seen': get('view_seen'),
        'view_year': get('view_year'),
        'view_runtime': get('view_runtime'),
        'view_rating': get('view_rating'),
        'view_created': get('view_created'),
        'view_updated': get('view_updated'),
        'rating_image': get('rating_image'),
        'spellchecker': get('spellchecker_pref'),
        'spell_notes': get('spell_notes'),
        'spell_plot': get('spell_plot'),
        'spell_lang': get('spell_lang'),
        'vbox_spellchecker': get('vbox_spellchecker'),
        'default_plugin': get('default_plugin'),
        'mail_smtp_server': get('mail_smtp_server'),
        'mail_use_auth': get('mail_use_auth'),
        'mail_username': get('mail_username'),
        'mail_password': get('mail_password'),
        'mail_email': get('mail_email'),
        'mail_smtp_port': get('mail_smtp_port'),
        'mail_use_tls': get('mail_use_tls'),
        'lang_name': get('lang_name_combo'),
        'tag_name': get('tag_name_combo'),
        'acodec_name': get('acodec_name_combo'),
        'achannel_name': get('achannel_name_combo'),
        'subformat_name': get('subformat_name_combo'),
        'medium_name': get('medium_name_combo'),
        'vcodec_name': get('vcodec_name_combo'),
        'sortby': get('p_sortby'),
        'sortby_reverse': get('p_sortby_reverse'),
    }
    self.widgets['preferences']['treeview'].connect('button_press_event', self.on_p_tree_button_press_event)
    self.widgets['preferences']['window'].connect('delete_event', self.on_delete_event_p)
    self.widgets['preferences']['spellchecker'].connect('toggled', self.on_cb_spellchecker_pref_toggled)
    self.widgets['preferences']['window'].set_transient_for(self.widgets['window'])
    #}}}

    self.widgets['results'] = {#{{{
        'window': get('results'),
        'treeview': get('results_treeview'),
        'select': get('results_select'),
        'cancel': get('results_cancel'),
    }
    self.widgets['results']['window'].connect('delete_event', self.on_results_window_close)
    self.widgets['results']['window'].set_transient_for(self.widgets['add']['window'])
    # default results window signals:
    self._resultswin_window_closed_signal = None
    self._resultswin_process = self.populate_dialog_with_results
    self._resultswin_button_pressed_signal = self.on_add_movie_results_button_press_event
    self.widgets['results']['select'].connect('clicked', self.on_results_select_press_event)
    self.widgets['results']['treeview'].connect('button_press_event', self.on_results_button_press_event)
    #}}}

    self.widgets['print_cover'] = {#{{{
        # TODO: merge these two windows
        'window_simple': get('w_print_cover_simple'),
        'window_image': get('w_print_cover_image'),
        'cs_size': get('cover_simple_size'),
        'cs_include_movie_number': get('cover_simple_include_movie_number'),
        'cs_include_poster': get('cover_simple_include_poster'),
        'ci_size': get('cover_image_size'),
        'ci_number': get('cover_image_number'),
    }
    self.widgets['print_cover']['window_simple'].connect('delete_event', self.on_delete_event_pcs)
    self.widgets['print_cover']['window_image'].connect('delete_event', self.on_delete_event_pci)
    self.widgets['print_cover']['window_simple'].set_transient_for(self.widgets['window'])
    self.widgets['print_cover']['window_image'].set_transient_for(self.widgets['window'])
    #}}}

    self.widgets['people'] = {#{{{
        'window': get('w_people'),
        'treeview': get('p_treeview'),
    }
    self.widgets['people']['window'].connect('delete_event', self.on_delete_event_wp)
    self.widgets['people']['window'].set_transient_for(self.widgets['window'])
    #}}}

    self.widgets['person'] = {#{{{
        # TODO: merge these two windows
        'window': get('w_add_person'),
        'e_window': get('w_edit_person'),
        'name': get('ap_name'),
        'email': get('ap_email'),
        'phone': get('ap_phone'),
        'e_name': get('ep_name'),
        'e_email': get('ep_email'),
        'e_phone': get('ep_phone'),
        'e_id': get('ep_id'),
    }
    self.widgets['person']['window'].connect('delete_event', self.on_delete_event_ap)
    self.widgets['person']['e_window'].connect('delete_event', self.on_delete_event_ep)
    self.widgets['person']['window'].set_transient_for(self.widgets['people']['window'])
    self.widgets['person']['e_window'].set_transient_for(self.widgets['people']['window'])
    #}}}

    self.widgets['filter'] = {#{{{
        'text': get('filter_txt'),
        'criteria': get('filter_criteria'),
        'collection': get('f_col'),
        'advfilter': get('f_advfilter'),
    }#}}}

    self.widgets['menu'] = {#{{{
        'preferences': get('preferences1'),
        'about': get('about1'),
        'menubar': get('menubar1'),
        'fullscreen': get('menu_fullscreen'),
        'toolbar': get('menu_toolbar'),
        'ext_toolbar': get('menu_ext_toolbar'),
        'export': get('export_menu'),
        'import': get('import_menu'),
        'not_seen_movies': get('seen_movies'),
        'loaned_movies': get('loaned_movies'),
        'all_movies': get('all_movies'),
        'loan': get('loan1'),
        'email': get('return1'),
        'return': get('e-mail_reminder1'),
        'select_all': get('select_all'),
        'clone' : get('clone'),
        'make_cover': get('make_cover'),
        'edit': get('edit1'),
    }#}}}

    self.widgets['popups'] = {#{{{
        'main': get('popup'),
        'loan': get('popup_loan'),
        'return': get('popup_return'),
        'email': get('popup_email'),
        'clone': get('clone1'),
        'print_cover': get('print_cover1'),
        'edit': get('edit2'),
    }#}}}
    self.widgets['extensions'] = {
        'toolbar': get('ext_toolbar'),
        'toolbar_hb': get('ext_toolbar_hb'),
        'preferences_vbox': get('p_extensions_vbox'),
    }

    self.widgets['w_loan_to'] = get('w_loan_to')
    self.widgets['w_loan_to'].connect('delete_event', self.on_delete_event_lt)

    self.widgets['poster_window'] = get('poster_window')
    self.widgets['poster_window'].connect('delete_event', self.on_delete_event_pw)
    self.widgets['poster_window'].set_transient_for(self.widgets['window'])
    self.widgets['big_poster'] = get('big_poster')

    #add some tooltips
    self.widgets['tooltips'] = gtk.Tooltips()
    self.widgets['tooltips'].set_tip(self.widgets['preferences']['epdf_reader'], _('Define here the PDF reader you want to use within Griffith. Popular choices are xpdf, gpdf, evince or kpdf. Make sure you have this program installed and working first.'))
    self.widgets['tooltips'].set_tip(self.widgets['preferences']['spell_lang'], _("Here you can define the desired language to use while spell checking some fields. Use you locale setting. For example, to use european portuguese spell checking enter 'pt'"))
    self.widgets['tooltips'].set_tip(self.widgets['preferences']['mail_smtp_server'], _("Use this entry to define the SMTP server you want to use to send e-mails. On *nix systems, 'localhost' should work. Alternatively, you can use your Internet Service Provider's SMTP server address."))
    self.widgets['tooltips'].set_tip(self.widgets['preferences']['mail_email'], _("This is the from e-mail address that should be used to all outgoing e-mail. You want to include your own e-mail address here probably."))
    self.widgets['tooltips'].set_tip(self.widgets['preferences']['mail_smtp_port'], _("Here you can define the smtp server port to use. If you want to use your gmail account service to send e-mails, try to enter '587' here. Default (and the most common one) is '25'"))
    self.widgets['tooltips'].set_tip(self.widgets['preferences']['mail_use_tls'], _("If you need TLS support, please check this checkbox. This is needed, for example, when using gmail service to send your reminder e-mails."))

    # define handlers for general events
    gladefile.signal_autoconnect({#{{{
        'gtk_main_quit': self.destroy,
        'on_window_state': self.on_window_state,
        'on_configure': self.on_configure,
        'on_about1_activate': self.about_dialog,
        'on_quit1_activate': self.destroy,
        'on_toolbar_quit_clicked': self.destroy,
        'on_toolbar_add_clicked': self.add_movie,
        'on_cancel_add_movie_clicked': self.hide_add_window,
        'on_add1_activate': self.add_movie,
        'on_add_movie_clicked': self.add_movie_db,
        'on_add_movie_close_clicked': self.add_movie_close_db,
        'on_delete_movie_clicked': self.delete_movie,
        'on_delete1_movie_activate': self.delete_movie,
        'on_main_treeview_row_activated': self.treeview_clicked,
        'on_row_activated': self.treeview_clicked,
        'on_get_from_web_clicked': self.get_from_web,
        'on_update_button_clicked': self.update_movie,
        'on_import_activate': self.on_import_activate,
        'on_seen_activate': self.toggle_seen,
        'on_select_all_activate': self.select_all,
        'on_fullscreen_activate': self.toggle_fullscreen,
        # preferences
        'on_preferences1_activate': self.show_preferences,
        'on_select_db_activate': lambda w: self.show_preferences(w, page=6),
        'on_cancel_preferences_clicked': self.hide_preferences,
        'on_save_preferences_clicked': self.save_preferences,
        'on_p_db_type_changed': self.on_p_db_type_changed,
        'on_backup_activate': self.backup,
        'on_restore_activate': self.restore,
        'on_merge_activate': self.merge,
        'on_cover_simple_activate': self.print_cover_simple_show,
        'on_cancel_print_cover_simple_clicked': self.print_cover_simple_hide,
        'on_b_print_cover_simple_clicked': self.print_cover_simple_process,
        'on_add_clear_clicked': self.clear_add_dialog,
        'on_am_save_button_clicked': self.update_movie,
        'on_people_activate': self.show_people_window,
        'on_cancel_people_clicked': self.hide_people_window,
        'on_filter_txt_changed': self.filter_txt,
        'on_filter_txt_key_release': self.filter_txt_key_release,
        'on_filter_criteria_changed': self.on_filter_criteria_changed,
        'on_clear_filter_clicked': self.clear_filter,
        'on_people_add_clicked': self.add_person,
        'on_add_person_cancel_clicked': self.add_person_cancel,
        'on_add_person_db_clicked': self.add_person_db,
        'on_people_delete_clicked': self.delete_person,
        'on_people_edit_clicked': self.edit_person,
        'on_edit_person_cancel_clicked': self.edit_person_cancel,
        'on_update_person_clicked': self.update_person,
        'on_clone_activate': self.clone_movie,
        'on_loan_button_clicked': self.loan_movie,
        'on_cancel_loan_clicked': self.cancel_loan,
        'on_loan_ok_clicked': self.commit_loan,
        'on_return_button_clicked': self.return_loan,
        'on_list_loaned_movies_activate': self.filter_loaned,
        'on_m_show_volume_button_clicked': self.show_volume,
        'on_m_show_collection_button_clicked': self.show_collection,
        'on_cover_choose_image_activate': self.print_cover_image,
        'on_cancel_print_cover_image_clicked': self.print_cover_image_hide,
        'on_b_print_cover_image_clicked': self.print_cover_image_process,
        'on_combo_source_changed': self.source_changed,
        # toolbar
        'on_view_toolbar_activate': self.toggle_toolbar,
        'on_view_ext_toolbar_activate': self.toggle_ext_toolbar,
        'on_go_first_clicked': self.go_first,
        'on_go_last_clicked': self.go_last,
        'on_go_back_clicked': self.go_prev,
        'on_go_forward_clicked': self.go_next,
        'on_new_bt_clicked': self.new_dbb,
        'on_new_activate': self.new_dbb,
        'on_edit_button_clicked': self.edit_movie,
        # poster
        'on_e_picture_clicked': self.change_poster,
        'on_open_poster_clicked': self.change_poster,
        'on_zoom_poster_clicked': self.z_poster,
        'on_delete_poster_clicked': self.del_poster,
        # URLs
        'on_goto_homepage_activate': self.on_goto_homepage_activate,
        'on_goto_forum_activate': self.on_goto_forum_activate,
        'on_goto_report_bug_activate': self.on_goto_report_bug_activate,
        'on_go_o_site_clicked': self.go_oficial_site,
        'on_go_site_clicked': self.go_site,
        'on_go_trailer_clicked': self.go_trailer_site,
        'on_seen_movies_activate': self.filter_not_seen,
        'on_all_movies_activate': self.filter_all,
        'on_rating_scale_add_value_changed': self.scale_rating_change_add,
        'on_sugest_activate': self.sugest_movie,
        'on_popup_delete_activate': self.delete_movie,
        'on_popup_clone_activate': self.clone_movie,
        'on_popup_simple_activate': self.print_cover_simple_show,
        'on_popup_choose_image_activate': self.print_cover_image,
        # loans
        'on_popup_loan_activate': self.loan_movie,
        'on_popup_return_activate': self.return_loan,
        'on_popup_email_activate': self.email_reminder,
        'on_email_reminder_clicked': self.email_reminder,
        # volumes/collections
        'on_am_collection_combo_changed': self.on_am_collection_combo_changed,
        'on_am_volume_combo_changed': self.on_am_volume_combo_changed,
        'on_am_add_volume_button_clicked': self.add_volume,
        'on_am_add_collection_button_clicked': self.add_collection,
        'on_am_remove_volume_button_clicked': self.remove_volume,
        'on_am_remove_collection_button_clicked': self.remove_collection,
        'on_f_col_changed': self.filter_collection,
        'on_f_advfilter_changed': self.filter_using_advfilter_rule,
        'on_results_cancel_clicked': self.on_results_window_close,
        # languages
        'on_lang_add_clicked': self.on_lang_add_clicked,
        'on_lang_remove_clicked': self.on_lang_remove_clicked,
        'on_am_lang_add_clicked': self.on_am_lang_add_clicked,
        'on_am_lang_remove_clicked': self.on_am_lang_remove_clicked,
        'on_lang_rename_clicked': self.on_lang_rename_clicked,
        'on_lang_name_combo_changed': self.on_lang_name_combo_changed,
        # tags
        'on_tag_add_clicked': self.on_tag_add_clicked,
        'on_tag_remove_clicked': self.on_tag_remove_clicked,
        'on_tag_rename_clicked': self.on_tag_rename_clicked,
        'on_tag_name_combo_changed': self.on_tag_name_combo_changed,
        # audio codecs
        'on_acodec_add_clicked': self.on_acodec_add_clicked,
        'on_acodec_remove_clicked': self.on_acodec_remove_clicked,
        'on_acodec_rename_clicked': self.on_acodec_rename_clicked,
        'on_acodec_name_combo_changed': self.on_acodec_name_combo_changed,
        # audio channels
        'on_achannel_add_clicked': self.on_achannel_add_clicked,
        'on_achannel_remove_clicked': self.on_achannel_remove_clicked,
        'on_achannel_rename_clicked': self.on_achannel_rename_clicked,
        'on_achannel_name_combo_changed': self.on_achannel_name_combo_changed,
        # subtitle formats
        'on_subformat_add_clicked': self.on_subformat_add_clicked,
        'on_subformat_remove_clicked': self.on_subformat_remove_clicked,
        'on_subformat_rename_clicked': self.on_subformat_rename_clicked,
        'on_subformat_name_combo_changed': self.on_subformat_name_combo_changed,
        # media
        'on_medium_add_clicked': self.on_medium_add_clicked,
        'on_medium_remove_clicked': self.on_medium_remove_clicked,
        'on_medium_rename_clicked': self.on_medium_rename_clicked,
        'on_medium_name_combo_changed': self.on_medium_name_combo_changed,
        # video codecs
        'on_vcodec_add_clicked': self.on_vcodec_add_clicked,
        'on_vcodec_remove_clicked': self.on_vcodec_remove_clicked,
        'on_vcodec_rename_clicked': self.on_vcodec_rename_clicked,
        'on_vcodec_name_combo_changed': self.on_vcodec_name_combo_changed,
        # advfilter
        'on_open_advfilterwindow_clicked': self.show_advfilter_window,
        'on_af_find_clicked': self.on_af_find_clicked,
        'on_advfilter_close': self.hide_advfilter_window,
        'on_advfilter_add': lambda w: advfilter.add_query_widget(self.widgets['advfilter']['dynamic_vbox'], self.field_names),
        'on_advfilter_clear': lambda w: advfilter.set_conditions(self.widgets['advfilter'], advfilter.get_def_conditions(), self.field_names),
        'on_advfilter_save': lambda w: advfilter.save(self.db, self.widgets['advfilter']),
        'on_advfilter_open': lambda w: advfilter.load(self.db, self.widgets['advfilter'], self.field_names),
        'on_advfilter_delete': lambda w: advfilter.delete(self.db, self.widgets['advfilter']),
    })#}}}


def populate_results_window(treemodel, items):
    treemodel.clear()

    if isinstance(items, dict):
        iterable = items.iteritems()
    else:
        iterable = items

    for key, value in iterable:
        myiter = treemodel.insert_before(None, None)
        treemodel.set_value(myiter, 0, str(key))
        treemodel.set_value(myiter, 1, value)



# -*- coding: UTF-8 -*-
# vim: fdm=marker
__revision__ = '$Id: __init__.py 1449 2010-09-29 21:03:04Z mikej06 $'
__version__ = 6 # XXX: database format version, remember to increase after changing data structures

# Copyright © 2009 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from sqlalchemy import MetaData, func, select, and_
from sqlalchemy.orm import mapper, relation, deferred, column_property, synonym

metadata = MetaData()
import tables # *after* metadata initialization
from _objects import *


mapper(Configuration, tables.configuration)
mapper(Volume, tables.volumes, order_by=tables.volumes.c.name, properties={
    'loaned': synonym('_loaned', map_column=True),
    'movies': relation(Movie, backref='volume')})
mapper(Collection, tables.collections, order_by=tables.collections.c.name, properties={
    'loaned': synonym('_loaned', map_column=True),
    'movies': relation(Movie, backref='collection')})
mapper(Medium, tables.media, properties={
    'movies': relation(Movie, backref='medium')})
mapper(Ratio, tables.ratios, properties={
    'movies': relation(Movie, backref='ratio')})
mapper(VCodec, tables.vcodecs, properties={
    'movies': relation(Movie, backref='vcodec')})
mapper(Person, tables.people, properties={
    'loans': relation(Loan, backref='person', cascade='all, delete-orphan'),
    'loaned_movies_count': column_property(select(
        [func.count(tables.loans.c.loan_id)],
        and_(tables.people.c.person_id == tables.loans.c.person_id,
             tables.loans.c.return_date == None))\
        .label('loaned_movies_count'), deferred=True),
    'returned_movies_count': column_property(select( # AKA loan history
        [func.count(tables.loans.c.loan_id)],
        and_(tables.people.c.person_id == tables.loans.c.person_id,
             tables.loans.c.return_date != None))\
        .label('returned_movies_count'), deferred=True)})
mapper(MovieLang, tables.movie_lang, primary_key=[tables.movie_lang.c.ml_id], properties={
    'movie': relation(Movie),
    'language': relation(Lang),
    'achannel': relation(AChannel),
    'acodec': relation(ACodec),
    'subformat': relation(SubFormat)})
mapper(ACodec, tables.acodecs, properties={
    'movielangs': relation(MovieLang)})
mapper(AChannel, tables.achannels, properties={
    'movielangs': relation(MovieLang)})
mapper(SubFormat, tables.subformats, properties={
    'movielangs': relation(MovieLang)})
mapper(Lang, tables.languages, properties={
    'movielangs': relation(MovieLang)})
mapper(MovieTag, tables.movie_tag)
mapper(Tag, tables.tags, properties={'movietags': relation(MovieTag, backref='tag')})
mapper(Loan, tables.loans, properties={
    'volume': relation(Volume),
    'collection': relation(Collection)})
mapper(Movie, tables.movies, order_by=tables.movies.c.number, properties={
    'loans': relation(Loan, backref='movie', cascade='all, delete-orphan'),
    #'tags': relation(Tag, cascade='all, delete-orphan', secondary=movie_tag,
    'tags': relation(Tag, secondary=tables.movie_tag,
                     primaryjoin=tables.movies.c.movie_id == tables.movie_tag.c.movie_id,
                     secondaryjoin=tables.movie_tag.c.tag_id == tables.tags.c.tag_id),
    'languages': relation(MovieLang, cascade='all, delete-orphan')})
mapper(Poster, tables.posters, properties={
    'movies': relation(Movie),
    'data': deferred(tables.posters.c.data)})
mapper(Filter, tables.filters)



# -*- coding: utf-8 -*-

__revision__ = '$Id: AMC.py 1605 2011-12-18 22:35:25Z piotrek $'

# Copyright (c) 2009

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from plugins.imp import ImportPlugin as IP
import os
import gutils
import string
import struct

import logging
log = logging.getLogger("Griffith")

# format:
#  LFFNR -> length field for next record
# field name        size in bytes
#--------------------------------
# Number            4
# Date              4
# Rating            4
# Year              4
# Length            4
# VideoBitrate      4
# AudioBitrate      4
# Disks             4
# Checked           1
# LFFNR             4
# Media             LFFNR
# LFFNR             4       # since V 3.3
# MediaType         LFFNR   # since V 3.3
# LFFNR             4       # since V 3.3
# MediaSource       LFFNR   # since V 3.3
# LFFNR             4
# Borrower          LFFNR
# LFFNR             4
# OriginalTitle     LFFNR
# LFFNR             4
# TranslatedTitle   LFFNR
# LFFNR             4
# Director          LFFNR
# LFFNR             4
# Producer          LFFNR
# LFFNR             4
# Country           LFFNR
# LFFNR             4
# Category          LFFNR
# LFFNR             4
# Actors            LFFNR
# LFFNR             4
# URL               LFFNR
# LFFNR             4
# Description       LFFNR
# LFFNR             4
# Comments          LFFNR
# LFFNR             4
# VideoFormat       LFFNR
# LFFNR             4
# AudioFormat       LFFNR
# LFFNR             4
# Resolution        LFFNR
# LFFNR             4
# Framerate         LFFNR
# LFFNR             4
# Languages         LFFNR
# LFFNR             4
# Subtitles         LFFNR
# LFFNR             4
# Size              LFFNR
# LFFNR             4
# Picture           LFFNR
# LFFNR             4
# PictureData       LFFNR


class ImportPlugin(IP):
    description  = 'Ant Movie Catalog (v3.5)'
    author       = 'Michael Jahn'
    email        = 'griffith@griffith.cc'
    version      = '1.1'
    file_filters = '*.[aA][mM][cC]'
    mime_types   = None

    fileHeader   = ' AMC_?.? Ant Movie Catalog         www.buypin.com  www.ant.be.tf '
    fileHeader31 = ' AMC_3.1 Ant Movie Catalog 3.1.x   www.buypin.com  www.ant.be.tf '
    fileHeader33 = ' AMC_3.3 Ant Movie Catalog 3.3.x   www.buypin.com  www.ant.be.tf '
    fileHeader35 = ' AMC_3.5 Ant Movie Catalog 3.5.x   www.buypin.com    www.antp.be '

    fileversion  = 0
    openfile     = None # for get_movie_details

    def initialize(self):
        if not IP.initialize(self):
            return False
        self.edit = False
        return True

    def set_source(self, name):
        IP.set_source(self, name)
        self.filename = name
        self.fileversion = self.read_fileversion()
        if self.fileversion == None:
            gutils.error(_('The format of the file is not supported.'))
            return False
        return True

    def count_movies(self):
        """Returns number of movies in file which is about to be imported"""
        count = 0
        nroffields = 23 # since V 3.3
        ifile = open(self.filename, 'rb')
        try:
            # seek to first record
            ifile.seek(len(self.fileHeader) + 16)
            # count records
            while True:
                ifile.seek(33, os.SEEK_CUR)
                i = 0
                while i < nroffields:
                    self.seekfield(ifile)
                    i = i + 1
                count = count + 1
        except EOFError:
            pass
        except Exception, e:
            log.error(str(e))
        finally:
            ifile.close()
        return count

    def get_movie_details(self):
        """Returns dictionary with movie details"""
        if self.openfile == None:
            self.openfile = open(self.filename, 'rb')
            # seek to first record
            self.openfile.seek(len(self.fileHeader) + 16)
        details = {}
        try:
            details['number'] = self.readintfield(self.openfile)        # Number            4
            self.openfile.seek(4, os.SEEK_CUR)                          # Date              4
            details['rating'] = self.readintfield(self.openfile)        # Rating            4
            details['year'] = self.readintfield(self.openfile)          # Year              4
            details['runtime'] = self.readintfield(self.openfile)       # Length            4
            self.openfile.seek(4, os.SEEK_CUR)                          # VideoBitrate      4
            self.openfile.seek(4, os.SEEK_CUR)                          # AudioBitrate      4
            details['media_num'] = self.readintfield(self.openfile)     # Disks             4
            details['seen'] = self.readboolfield(self.openfile)         # Checked           1
            self.seekfield(self.openfile)                               # Media             LFFNR
            if self.fileversion >= 3.3:
                # setting medium_id to string mediumname; mapping is done in base class
                details['medium_id'] = self.readstringfield(self.openfile) # MediaType         LFFNR   # since V 3.3
                self.seekfield(self.openfile)                           # MediaSource       LFFNR   # since V 3.3
            self.seekfield(self.openfile)                               # Borrower          LFFNR
            details['o_title'] = self.readstringfield(self.openfile)    # OriginalTitle     LFFNR
            details['title'] = self.readstringfield(self.openfile)      # TranslatedTitle   LFFNR
            details['director'] = self.readstringfield(self.openfile)   # Director          LFFNR
            self.seekfield(self.openfile)                               # Producer          LFFNR
            details['country'] = self.readstringfield(self.openfile)    # Country           LFFNR
            details['genre'] = self.readstringfield(self.openfile)      # Category          LFFNR
            details['cast'] = self.readstringfield(self.openfile)       # Actors            LFFNR
            details['o_site'] = self.readstringfield(self.openfile)     # URL               LFFNR
            details['plot'] = self.readstringfield(self.openfile)       # Description       LFFNR
            details['notes'] = self.readstringfield(self.openfile)      # Comments          LFFNR
            self.seekfield(self.openfile)                               # VideoFormat       LFFNR
            self.seekfield(self.openfile)                               # AudioFormat       LFFNR
            self.seekfield(self.openfile)                               # Resolution        LFFNR
            self.seekfield(self.openfile)                               # Framerate         LFFNR
            self.seekfield(self.openfile)                               # Languages         LFFNR
            self.seekfield(self.openfile)                               # Subtitles         LFFNR
            self.seekfield(self.openfile)                               # Size              LFFNR
            posterfilename = self.readstringfield(self.openfile)        # Picture           LFFNR
            posterdata = self.readbinaryfield(self.openfile)            # PictureData       LFFNR
            if posterdata:
                details['poster'] = posterdata
            elif posterfilename and len(posterfilename) > 4:
                details['poster'] = posterfilename

            if details['title'] == None:
                details['title'] = details['o_title']
            if details['o_title'] == None:
                details['o_title'] = details['title']
            if not details['cast'] == None:
                details['cast'] = string.replace(details['cast'], ', ', '\n')
            if self.fileversion >= 3.5:
                if not details['rating'] == None:
                    details['rating'] = round(float(details['rating']) / 10.0, 0)
        except EOFError:
            details = None
        except Exception, e:
            log.error(str(e))
            details = None
        return details

    def clear(self):
        """clear plugin before next source file"""
        IP.clear(self)
        if self.openfile:
            self.openfile.close()
            self.openfile = None
            self.fileversion = 0

    def destroy(self):
        """close all resources"""
        IP.destroy(self)

    def read_fileversion(self):
        version = None
        header = ''
        ifile = open(self.filename, 'rb')
        try:
            header = ifile.read(len(self.fileHeader))
        finally:
            ifile.close()
        # no support for other versions at the moment
        if header == self.fileHeader35:
            version = 3.5
        elif header == self.fileHeader33:
            version = 3.3
        elif header == self.fileHeader31:
            version = 3.1
        return version

    def seekfield(self, ifile):
        lenStr = ifile.read(4)
        if len(lenStr) < 4:
            raise EOFError
        lenTuple = struct.unpack('i', lenStr)
        if lenTuple[0]:
            ifile.seek(lenTuple[0], os.SEEK_CUR)

    def readstringfield(self, ifile):
        field = None
        lenStr = ifile.read(4)
        if len(lenStr) < 4:
            raise EOFError
        lenTuple = struct.unpack('i', lenStr)
        if lenTuple[0]:
            field = ifile.read(lenTuple[0]).decode('iso8859-1')
        return field

    def readintfield(self, ifile):
        intStr = ifile.read(4)
        if len(intStr) < 4:
            raise EOFError
        intTuple = struct.unpack('i', intStr)
        if intTuple[0] == -1:
            return None
        return intTuple[0]

    def readboolfield(self, ifile):
        byteStr = ifile.read(1)
        intValue = ord(byteStr[0])
        if intValue == -1:
            return None
        return intValue != 0

    def readbinaryfield(self, ifile):
        field = None
        lenStr = ifile.read(4)
        if len(lenStr) < 4:
            raise EOFError
        lenTuple = struct.unpack('i', lenStr)
        if lenTuple[0]:
            field = ifile.read(lenTuple[0])
        return field



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2007-2011
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string
import re

plugin_name         = "Filmtipset.se"
plugin_description  = "Filmtipset.se"
plugin_url          = "www.filmtipset.se"
plugin_language     = _("Swedish")
plugin_author       = "Michael Jahn"
plugin_author_email = "<mikej06@hotmail.com>"
plugin_version      = "1.2"

class Plugin(movie.Movie):

    def __init__(self, id):
        self.encode   = 'iso-8859-1'
        self.movie_id = id
        self.url      = 'http://www.filmtipset.se/film/' + str(self.movie_id)

    def get_image(self):
        self.image_url = ''
        tmp = gutils.trim(self.page, 'src="http://images.filmtipset.se/posters/', '"')
        if tmp != '':
            self.image_url = 'http://images.filmtipset.se/posters/' + tmp

    def get_o_title(self):
        self.o_title = gutils.trim(self.page, 'Originaltitel:', '</tr>')
        if self.o_title == '':
            self.o_title = gutils.trim(self.page, '<h1>', '</h1>')

    def get_title(self):
        self.title = gutils.after(gutils.trim(self.page, 'class="movie_header"', '<td'), '>')

    def get_director(self):
        self.director = gutils.trim(self.page, 'Regiss&ouml;r:', '</tr>')

    def get_plot(self):
        self.plot = gutils.regextrim(self.page, '<h2>Om [^>]*[>]', '</tr>')

    def get_year(self):
        self.year = gutils.trim(self.page, 'Utgivnings&aring;r:', '</tr>')

    def get_runtime(self):
        self.runtime = string.strip(gutils.trim(self.page, 'L&auml;ngd:', ' min'))

    def get_genre(self):
        self.genre = string.strip(gutils.trim(self.page, 'Nyckelord:', '</tr>'))
        if self.genre == '':
            self.genre = string.strip(gutils.trim(self.page, 'Genre:', '</tr>'))

    def get_cast(self):
        self.cast = gutils.regextrim(self.page, 'Sk&aring;despelare[^:]*[:]', '</tr>')
        self.cast = string.replace(self.cast, ', ', '\n')

    def get_classification(self):
        self.classification = ''

    def get_studio(self):
        self.studio = ''

    def get_o_site(self):
        self.o_site = ''
        tmp = gutils.trim(self.page, 'http://www.imdb.com', '"')
        if tmp != '':
            self.o_site = 'http://www.imdb.com' + tmp

    def get_site(self):
        self.site = self.url

    def get_trailer(self):
        self.trailer = ""

    def get_country(self):
        self.country = string.strip(gutils.trim(self.page, 'Produktionsland:', '</span>'))

    def get_rating(self):
        self.rating = "0"
        tmp = gutils.trim(self.page, 'grade_images/grade1', '_')
        try:
            self.rating = str(int(tmp) * 2)
        except:
            pass

    def get_notes(self):
        self.notes = ''
        tmp = gutils.trim(self.page, 'Alt. titel:', '</span>')
        if tmp != '':
            self.notes = self.notes + 'Alt. titel:' + string.strip(gutils.strip_tags(tmp))

    def get_screenplay(self):
        self.screenplay = gutils.trim(self.page, 'Manus:', '</tr>')

class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        self.original_url_search   = "http://www.filmtipset.se/search.cgi?field=name&field=orgname&show_graded=1&search_value="
        self.translated_url_search = "http://www.filmtipset.se/search.cgi?field=name&field=orgname&show_graded=1&search_value="
        self.encode                = 'iso-8859-1'

    def search(self,parent_window):
        if not self.open_search(parent_window):
            return None
        tmp_page = gutils.trim(self.page, 'Matchning', 'Hittade')
        if tmp_page == '':
            tmp_page = gutils.trim(self.page, 'Matchning', 'Visa fler')
        self.page = tmp_page
        return self.page

    def get_searches(self):
        elements1 = re.split('href="film/', self.page)
        elements1[0] = None
        for element in elements1:
            if element <> None:
                searchResult = re.search('["&?]', element)
                if searchResult is None:
                    self.ids.append(gutils.before(element, '"'))
                else:
                    self.ids.append(element[:searchResult.end() - 1])
                self.titles.append(
                    gutils.strip_tags(gutils.trim(element, '>', '</a>')) + ' / ' +
                    gutils.clean(string.replace(gutils.trim(element, 'Originaltitel:', '</div>'), '&nbsp;', ' '))
                )

#
# Plugin Test
#
class SearchPluginTest(SearchPlugin):
    #
    # Configuration for automated tests:
    # dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
    #
    test_configuration = {
        'Rocky' : [ 19, 19 ],
    }

class PluginTest:
    #
    # Configuration for automated tests:
    # dict { movie_id -> dict { arribute -> value } }
    #
    # value: * True/False if attribute only should be tested for any value
    #        * or the expected value
    #
    test_configuration = {
        'rocky-balboa.html' : {
            'title'               : 'Rocky Balboa',
            'o_title'             : 'Rocky Balboa',
            'director'            : 'Sylvester Stallone',
            'plot'                : True,
            'cast'                : 'Sylvester Stallone\n\
Burt Young\n\
Milo Ventimiglia\n\
Geraldine Hughes\n\
Antonio Tarver\n\
James Francis Kelly III\n\
Tony Burton\n\
Henry G Sanders',
            'country'             : 'USA',
            'genre'               : 'Drama, Action',
            'classification'      : False,
            'studio'              : False,
            'o_site'              : 'http://www.imdb.com/title/tt0479143/',
            'site'                : 'http://www.filmtipset.se/film/rocky-balboa.html',
            'trailer'             : False,
            'year'                : 2006,
            'notes'               : False,
            'runtime'             : 102,
            'image'               : True,
            'rating'              : 6,
            'cameraman'           : False,
            'screenplay'          : 'Sylvester Stallone',
            'barcode'             : False
        },
    }



# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovieKinoDe.py 1599 2011-12-02 19:52:02Z mikej06 $'

# Copyright (c) 2006-2011
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import string
import re

plugin_name         = "Kino.de"
plugin_description  = "KINO.DE"
plugin_url          = "www.kino.de"
plugin_language     = _("German")
plugin_author       = "Michael Jahn"
plugin_author_email = "<mikej06@hotmail.com>"
plugin_version      = "1.16"

class Plugin(movie.Movie):
    url_to_use_base = 'http://www.kino.de/'
    url_to_use      = url_to_use_base + 'kinofilm/'
    url_type        = 'K'

    def __init__(self, id):
        self.encode='iso-8859-1'
        elements = string.split(id, "_")
        self.movie_id = elements[1]
        if (elements[0] == "V"):
            self.url_to_use_base = 'http://www.video.de/'
            self.url_to_use      = self.url_to_use_base + 'videofilm/'
            self.url_type        = 'V'
        else:
            self.url_to_use_base = 'http://www.kino.de/'
            self.url_to_use      = self.url_to_use_base + 'kinofilm/'
            self.url_type        = 'K'
        self.url = self.url_to_use + str(self.movie_id)

    def initialize(self):
        if self.url_type == 'K':
            url = self.url_to_use + string.replace(str(self.movie_id), '/', '/credits/')
            self.creditspage = self.open_page(self.parent_window, url=url)
        else:
            self.creditspage = ''
        videopageforkinourl = gutils.trim(self.page, 'class="videode"', 'Zum Film auf video.de')
        if videopageforkinourl:
            url = gutils.trim(videopageforkinourl, 'href="', '"')
            self.videopage = self.open_page(self.parent_window, url=url)
        else:
            self.videopage = None

    def get_image(self):
        self.image_url = ''
        tmpdata = gutils.regextrim(self.page, '<div class="cover-area">', '</div>')
        if tmpdata:
            # video page
            tmpdata = re.search('(http[:][/][/][^/]+[/]flbilder[/][^"\']+)', tmpdata)
            if tmpdata:
                self.image_url = tmpdata.group(1)
        else:
            # kino page
            tmpdata = gutils.before(self.page, '<span style="line-height: 15px;">')
            if tmpdata:
                tmpparts = re.split('http://images.kino.de/s/', tmpdata)
                if len(tmpparts) > 2:
                    self.image_url = 'http://images.kino.de/s/' + gutils.before(tmpparts[2], '"')
                elif len(tmpparts) > 1:
                    self.image_url = 'http://images.kino.de/s/' + gutils.before(tmpparts[1], '"')
        if not self.image_url and self.videopage:
            tmpdata = gutils.regextrim(self.videopage, '<div class="cover-area">', '</div>')
            if tmpdata:
                # video page
                tmpdata = re.search('(http[:][/][/][^/]+[/]flbilder[/][^"\']+)', tmpdata)
                if tmpdata:
                    self.image_url = tmpdata.group(1)


    def get_o_title(self):
        self.o_title = gutils.trim(self.page, '<p>Originaltitel: ', '</p>')
        if not self.o_title:
            self.o_title = gutils.trim(self.page, '<h1(', ')')
            if not self.o_title:
                self.o_title = gutils.trim(self.page, '<div class="teaser">', '</')
                if not self.o_title:
                    if self.videopage:
                        self.o_title = gutils.trim(self.videopage, '<p>Originaltitel: ', '</p>')
                    if not self.o_title:
                        self.o_title = gutils.regextrim(self.page, '<h1>', '</h1>')

    def get_title(self):
        self.title = gutils.trim(self.page, '<div class="teaser">', '</')
        if not self.title:
            self.title = gutils.regextrim(self.page, '<h1>', '</h1>')

    def get_director(self):
        self.director = gutils.trim(self.page, '<th>Regie:', '<th>')
        if not self.director:
            self.director = gutils.trim(self.creditspage, 'Regie&nbsp;', '</tr>')

    def get_plot(self):
        self.plot = gutils.trim(self.page, '<div class="yui-content">', '<div class="footer">')
        if not self.plot:
            # kino page
            self.plot = gutils.trim(self.page, '<span style="line-height: 15px;">', '<table')
        if not self.plot and self.videopage:
            self.plot = gutils.trim(self.videopage, '<div class="yui-content">', '<div class="footer">')
        if self.plot:
            # video page
            self.plot = re.sub('<script type="text/javascript">[^<]+</script>', '', self.plot)
            self.plot = string.replace(self.plot, '>Großansicht</a>', '>')
            self.plot = string.replace(self.plot, '>Schließen</a>', '>')
            self.plot = string.replace(self.plot, '>zur&uuml;ck </a>', '>')
            self.plot = string.replace(self.plot, '>1</a>', '>')
            self.plot = string.replace(self.plot, '> weiter</a>', '>')
            self.plot = string.replace(self.plot, '</h4>', '\n')
            self.plot = gutils.clean(self.plot)
            compiledmultiline = re.compile(r'^[^(]+[(]Foto[:][^)]+[)][ ]*$', re.MULTILINE)
            self.plot = compiledmultiline.sub('', self.plot)
            compiledmultiline = re.compile(r"(^\s+$|^\s*//\s*$)", re.MULTILINE)
            self.plot = compiledmultiline.sub('', self.plot)
            compiledmultiline = re.compile("^[\n]+$", re.MULTILINE)
            self.plot = compiledmultiline.sub("\n", self.plot)

    def get_year(self):
        self.year = ''
        tmp = gutils.trim(self.page, '<div class="description">', '</div>')
        if tmp:
            searchyearandcountry = re.search('([0-9]{4})<br', tmp)
            if searchyearandcountry:
                self.year = searchyearandcountry.group(1)
        if not self.year:
            tmp = gutils.trim(self.page, '<span class="standardsmall"><strong>', '<br')
            if tmp:
                tmp = gutils.trim(tmp, '<strong>', '</strong>')
                if tmp:
                    srchyear = re.search('([0-9]{4})', tmp)
                    if srchyear:
                        self.year = srchyear.group(1)
        if not self.year and self.videopage:
            tmp = gutils.trim(self.videopage, '<div class="description">', '</div>')
            if tmp:
                searchyearandcountry = re.search('([0-9]{4})<br', tmp)
                if searchyearandcountry:
                    self.year = searchyearandcountry.group(1)

    def get_runtime(self):
        self.runtime = ''
        srchresult = re.search('Laufzeit: ([0-9]+)[ \t]Min[.]<', self.page)
        if srchresult <> None:
            self.runtime = srchresult.group(1)
        if not self.runtime:
            srchresult = re.search('>([0-9]+)[ \t]Min[.]<', self.page)
            if srchresult <> None:
                self.runtime = srchresult.group(1)
        if not self.runtime and self.videopage:
            srchresult = re.search('Laufzeit: ([0-9]+)[ \t]Min[.]<', self.videopage)
            if srchresult <> None:
                self.runtime = srchresult.group(1)

    def get_genre(self):
        self.genre = gutils.trim(self.page,'<p class="genre">', '</p>')
        if not self.genre:
            self.genre = gutils.trim(self.page, 'title="Zur Genreliste: Drama">', '<')
        if not self.genre and self.videopage:
            self.genre = gutils.trim(self.videopage,'<p class="genre">', '</p>')

    def get_cast(self):
        self.cast = ''
        tmp = gutils.regextrim(self.page, '<th>Darsteller:', '(<th>[^&]|</table>)')
        if tmp:
            tmpparts = string.split(tmp, '<a href="/star/')
            for tmppart in tmpparts[1:]:
                name = gutils.trim(tmppart, '>', '<')
                role = gutils.trim(tmppart, '>als ', '<')
                if name:
                    if role:
                        self.cast = self.cast + name + _(' as ') + role + '\n'
                    else:
                        self.cast = self.cast + name + '\n'
        if not self.cast:
            tmp = gutils.trim(self.creditspage, '>Cast<br />', '>Crew<')
            if tmp:
                castparts = re.split('width="50%"><a href="/star/', tmp)
                for index in range(1, len(castparts), 1):
                    role = gutils.clean(gutils.trim(castparts[index - 1], 'width="50%">', '</td>'))
                    name = gutils.clean(gutils.trim(castparts[index], '">', '<'))
                    if role:
                        self.cast = self.cast + name + _(' as ') + role + '\n'
                    else:
                        self.cast = self.cast + name + '\n'

    def get_classification(self):
        self.classification = gutils.regextrim(self.page, 'FSK: ', '<')
        if not self.classification and self.videopage:
            self.classification = gutils.regextrim(self.videopage, 'FSK: ', '<')

    def get_studio(self):
        self.studio = ''
        tmp = gutils.trim(self.page, '<div class="description">', '</div>')
        if tmp:
            tmp = gutils.trim(tmp, 'Regie:', '</p>')
            if tmp:
                self.studio = string.replace(gutils.after(tmp, '<br/>'), 'Verleih: ', '')
        if not self.studio:
            self.studio = gutils.trim(self.page, 'Verleih: ', '<')
        if not self.studio and self.videopage:
            tmp = gutils.trim(self.videopage, '<div class="description">', '</div>')
            if tmp:
                tmp = gutils.trim(tmp, 'Regie:', '</p>')
                if tmp:
                    self.studio = string.replace(gutils.after(tmp, '<br/>'), 'Verleih: ', '')

    def get_o_site(self):
        self.o_site = ""

    def get_site(self):
        self.site = self.url_to_use + self.movie_id

    def get_trailer(self):
        self.trailer = ''
        trailerparts = re.split('href="/trailer-und-bilder/film', self.page)
        if len(trailerparts) > 1:
            for trailerpart in trailerparts[1:]:
                trailermatch = re.search('Trailer[ ]*</p>', trailerpart)
                if trailermatch:
                    self.trailer = self.url_to_use_base + 'trailer-und-bilder/film' + gutils.before(trailerpart, '"')
                    break
        if not self.trailer and self.url_type == 'K':
            self.trailer = self.url_to_use + string.replace(str(self.movie_id), '/', '/trailer/')

    def get_country(self):
        self.country = ''
        tmp = gutils.trim(self.page, '<div class="description">', '</div>')
        if tmp:
            searchyearandcountry = re.search('([^>0-9]+)[0-9]{4}<br', tmp)
            if searchyearandcountry:
                self.country = searchyearandcountry.group(1)
        if not self.country:
            tmp = gutils.trim(self.page, '<span class="standardsmall"><strong>', '<br')
            if tmp:
                tmp = gutils.trim(tmp, '<strong>', '</strong>')
                if tmp:
                    self.country = gutils.before(tmp, ' ')
        if not self.country and self.videopage:
            tmp = gutils.trim(self.videopage, '<div class="description">', '</div>')
            if tmp:
                searchyearandcountry = re.search('([^>0-9]+)[0-9]{4}<br', tmp)
                if searchyearandcountry:
                    self.country = searchyearandcountry.group(1)

    def get_rating(self):
        self.rating = 0
        tmp = gutils.trim(self.page, '<h4>Filmbewertung</h4>', '</script>')
        if tmp:
            matched = re.search('ratingBar.setValue[(]([0-9]+)[)]', tmp)
            if matched:
                try:
                    self.rating = round(int(matched.group(1)) / 10.0, 0)
                except:
                    pass

    def get_notes(self):
        self.notes = ""
        tmp_notes = gutils.clean(gutils.trim(self.page, "<strong>Sprachen:</strong>", "</p>"))
        if tmp_notes != "":
            self.notes = self.notes + "Sprachen:\n" + tmp_notes + "\n\n"
        tmp_notes = gutils.clean(gutils.trim(self.page, "<strong>Untertitel:</strong>", "</p>"))
        if tmp_notes != "":
            self.notes = self.notes + "Untertitel:\n" + tmp_notes + "\n\n"
        tmp_notes = gutils.clean(gutils.trim(self.page, "<strong>Tonformat:</strong>", "</p>"))
        if tmp_notes != "":
            self.notes = self.notes + "Tonformat:\n" + tmp_notes + "\n\n"
        tmp_notes = gutils.clean(gutils.trim(self.page, "<strong>Bildformat:</strong>", "</p>"))
        if tmp_notes != "":
            self.notes = self.notes + "Bildformat:\n" + tmp_notes + "\n\n"
        tmp_notes = gutils.clean(gutils.trim(self.page, "<strong>EAN</strong>", "</p>"))
        if tmp_notes != "":
            self.notes = self.notes + "EAN:\n" + tmp_notes + "\n\n"

    def get_screenplay(self):
        self.screenplay = gutils.regextrim(self.page, '<th>Buch:', '<th>')
        if not self.screenplay:
            self.screenplay= gutils.trim(self.creditspage, 'Drehbuch:&nbsp;', '</tr>')

    def get_cameraman(self):
        self.cameraman = gutils.regextrim(self.page, '<th>Kamera:', '(<th>|</table>)')
        if not self.cameraman:
            self.cameraman= gutils.trim(self.creditspage, 'Kamera&nbsp;', '</tr>')

class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        self.original_url_search   = 'http://www.kino.de/suche/film?hitsPerPage=50&searchString='
        self.translated_url_search = 'http://www.kino.de/suche/film?hitsPerPage=50&searchString='
        self.encode='iso-8859-1'
        self.remove_accents = False

    def search(self,parent_window):
        self.open_search(parent_window)
        pagemovie = self.page
        #
        # Look for DVD and VHS
        #
        self.url = "http://www.kino.de/suche/video?hitsPerPage=50&searchString="
        self.open_search(parent_window)
        self.page = pagemovie + self.page

        return self.page

    def get_searches(self):
        elements1 = re.split('href="/kinofilm/', self.page)
        elements1[0] = None
        for element in elements1:
            if element <> None:
                title = gutils.clean(gutils.trim(element,'>','</a>')) + string.replace(' (' +
                            gutils.clean(gutils.trim(element, '<p>', "<br />")) + ')', '()', '')
                if title != ' ':
                    self.ids.append("K_" + re.sub('[?].*', '', gutils.before(element,'"')))
                    self.titles.append('Kino: ' + title)

        elements2 = re.split('href="http://www.video.de/videofilm/', self.page)
        elements2[0] = None
        for element in elements2:
            if element <> None:
                title = gutils.clean(gutils.trim(element,'>','</a>')) + string.replace(' (' +
                            gutils.clean(gutils.trim(element, '<p>', "<br />")) + ')', '()', '')
                if title != ' ':
                    id = re.sub('[?].*', '', gutils.before(element,'"'))
                    self.ids.append("V_" + id)
                    type = ''
                    if 'blu-ray-disc-kauf' in id:
                        type = ' (Bluray-Kauf)'
                    if 'blu-ray-disc-leih' in id:
                        type = ' (Bluray-Verleih)'
                    if 'dvd-leih' in id:
                        type = ' (DVD-Verleih)'
                    if 'dvd-kauf' in id:
                        type = ' (DVD-Kauf)'
                    self.titles.append('Video: ' + title + type)

#
# Plugin Test
#
class SearchPluginTest(SearchPlugin):
    #
    # Configuration for automated tests:
    # dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }
    #
    test_configuration = {
        'Rocky Balboa'         : [ 9, 9 ],
        'Arahan'               : [ 10, 10 ],
        'Ein glückliches Jahr' : [ 4, 4 ]
    }

class PluginTest:
    #
    # Configuration for automated tests:
    # dict { movie_id -> dict { arribute -> value } }
    #
    # value: * True/False if attribute only should be tested for any value
    #        * or the expected value
    #
    test_configuration = {
        'K_rocky-balboa/96132.html' : {
            'title'               : 'Rocky Balboa',
            'o_title'             : 'Rocky Balboa',
            'director'            : 'Sylvester Stallone',
            'plot'                : True,
            'cast'                : 'Sylvester Stallone' + _(' as ') + 'Rocky Balboa\n\
Antonio Traver' + _(' as ') + 'Mason "The Line" Dixon\n\
Burt Young' + _(' as ') + 'Paulie\n\
Geraldine Hughes' + _(' as ') + 'Marie\n\
Milo Ventimiglia' + _(' as ') + 'Rocky Jr.\n\
James Francis Kelly III' + _(' as ') + 'Steps\n\
Tony Burton' + _(' as ') + 'Duke\n\
A.J. Benza' + _(' as ') + 'L.C.',
            'country'             : 'USA',
            'genre'               : 'Drama',
            'classification'      : 'ab 12 Jahre',
            'studio'              : 'Fox',
            'o_site'              : False,
            'site'                : 'http://www.kino.de/kinofilm/rocky-balboa/96132.html',
            'trailer'             : 'http://www.kino.de/kinofilm/rocky-balboa/trailer/96132.html',
            'year'                : 2006,
            'notes'               : False,
            'runtime'             : 102,
            'image'               : True,
            'rating'              : False,
            'cameraman'           : 'J. Clark Mathis',
            'screenplay'          : 'Sylvester Stallone'
        },
        'K_ein-glueckliches-jahr/28675.html' : {
            'title'               : 'Ein glückliches Jahr',
            'o_title'             : 'La bonne année',
            'director'            : 'Claude Lelouch',
            'plot'                : True,
            'cast'                : 'Lino Ventura\n\
Françoise Fabian\n\
Charles Gérard\n\
André Falcon',
            'country'             : 'Frankreich/Italien',
            'genre'               : 'Drama',
            'classification'      : 'ab 12',
            'studio'              : 'Black Hill Pictures',
            'o_site'              : False,
            'site'                : 'http://www.kino.de/kinofilm/ein-glueckliches-jahr/28675.html',
            'trailer'             : 'http://www.kino.de/kinofilm/ein-glueckliches-jahr/trailer/28675.html',
            'year'                : 1973,
            'notes'               : False,
            'runtime'             : 110,
            'image'               : True,
            'rating'              : False,
            'cameraman'           : 'Jean Collomb',
            'screenplay'          : 'Claude Lelouch'
        },
        'V_ein-glueckliches-jahr-dvd/85546.html' : {
            'title'               : 'Ein glückliches Jahr',
            'o_title'             : 'La bonne année',
            'director'            : 'Claude Lelouch',
            'plot'                : True,
            'cast'                : 'Lino Ventura\n\
Françoise Fabian\n\
Charles Gérard\n\
André Falcon',
            'country'             : 'Frankreich/Italien',
            'genre'               : 'Drama',
            'classification'      : 'ab 12',
            'studio'              : 'Black Hill Pictures',
            'o_site'              : False,
            'site'                : 'http://www.video.de/videofilm/ein-glueckliches-jahr-dvd/85546.html',
            'trailer'             : False,
            'year'                : 1973,
            'notes'               : 'Sprachen:\n\
Deutsch DD 2.0, Französisch DD 2.0\n\
\n\
Tonformat:\n\
Dolby Digital 2.0\n\
\n\
Bildformat:\n\
1:1,33/4:3',
            'runtime'             : 110,
            'image'               : True,
            'rating'              : False,
            'cameraman'           : 'Jean Collomb',
            'screenplay'          : 'Claude Lelouch'
        },
        'V_arahan-vanilla-dvd/90405.html' : {
            'title'               : 'Arahan',
            'o_title'             : 'Arahan jangpung dae jakjeon',
            'director'            : 'Ryoo Seung-wan',
            'plot'                : True,
            'cast'                : 'Ryu Seung-beom' + _(' as ') + 'Sang-hwan\n\
Yoon So-yi' + _(' as ') + 'Wi-jin\n\
Ahn Sung-kee' + _(' as ') + 'Ja-woon\n\
Jung Doo-hong' + _(' as ') + 'Heuk-woon\n\
Yun Ju-sang' + _(' as ') + 'Mu-woon',
            'country'             : 'Südkorea',
            'genre'               : 'Action/ Komödie',
            'classification'      : 'ab 16',
            'studio'              : 'Splendid Film',
            'o_site'              : False,
            'site'                : 'http://www.video.de/videofilm/arahan-vanilla-dvd/90405.html',
            'trailer'             : False,
            'year'                : 2004,
            'notes'               : 'Sprachen:\n\
Deutsch DD 5.1\n\
\n\
Tonformat:\n\
Dolby Digital 5.1\n\
\n\
Bildformat:\n\
1:1,78/16:9',
            'runtime'             : 108,
            'image'               : True,
            'rating'              : False,
            'cameraman'           : 'Lee Jun-gyu',
            'screenplay'          : 'Ryoo Seung-wan'
        }
    }
